--[[
$Id: Options.lua 50 2012-03-13 01:15:07Z stassart $

Author: Trelis @ Proudmoore
(pally (a) stassart o org)

Copyright 2010-2012 Benjamin Stassart

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

]]

-- Prevent LUA error with blank WTF
-- Disable PallyPowerWarn if the player is not a paladin
if select(2, UnitClass('player')) ~= "PALADIN" then
	DisableAddOn("PallyPowerWarn")
	return
end

local L = LibStub("AceLocale-3.0"):GetLocale("PallyPowerWarn")

PallyPowerWarn.options = {
	name = "PallyPower Warnings",
	handler = PallyPowerWarn,
	type ='group',
	childGroups ='tree',
	desc = L["Options for PallyPowerWarn"],
	args = {
		ui = {
			name = L["Show UI"],
			type = "execute",
			desc = L["Shows the Graphical User Interface"],
			func = "OpenConfig",
			guiHidden = true,
			order = 999,
		},
		version = {
			name = L["Show version"],
			type = "execute",
			desc = L["Show version"],
			func = "ShowVersion",
			guiHidden = true,
			order = 998,
		},
		check = {
			name = L["Perform checks"],
			type = "execute",
			desc = L["Perform checks"],
			func = "PerformChecks",
			guiHidden = true,
			order = 997,
		},
		display = {
			name = L["General Display"],
			type = 'group',
			desc = L["General Display settings and options for the Custom Message Frame."],
			order = 100, 
			args = {
				info = {
					type = "description",
					order = 101,
					name = "Version " .. PallyPowerWarn.DisplayVersion .. "\n\nThis addon provides warnings for Blessings, Seals, Righteous Fury, and Auras.",
				},
				chatheader = {
					order = 109,
					type = "header",
					name = L["Chat Window Options"],
				},
				chat = {
					name = L["Chat Message"],
					type = "toggle",
					desc = L["Display message in Chat Frame."],
					get = function(info) return PallyPowerWarn.db.profile.display.chat end,
					set = function(info,v) PallyPowerWarn.db.profile.display.chat = v end,
					order = 110,
				},
				number= {
					name = L["Chat number"],
					type = "range",
					desc = L["Choose which chat to display the messages in (0=default)."],
					get = function(info) return PallyPowerWarn.db.profile.display.number end,
					set = function(info,v)
						PallyPowerWarn.db.profile.display.number = v
					end,
					step = 1,
					min = 0,
					max = 10,
					order = 111,
				},
				frameheader = {
					name = L["Message Frame"],
					type = "header",
					order = 112
				},
				-- error = {
				-- 	name = L["Error Frame"],
				-- 	type = "toggle",
				-- 	desc = L["Display message in Blizzard UI Error Frame."],
				-- 	get = function(info) return PallyPowerWarn.db.profile.display.error end,
				-- 	set = function(info,v) PallyPowerWarn.db.profile.display.error = v end,
				-- 	order = 120,
				-- },
				-- h121 = {type = "header", name = " ", order = 121},
				frame = {
					name = L["Message Frame"],
					type = "toggle",
					desc = L["Display message in Custom Message Frame."],
					get = function(info) return PallyPowerWarn.db.profile.display.frame end,
					set = function(info,v)
						PallyPowerWarn.db.profile.display.frame = v
						PallyPowerWarn:LoadEvents()
					end,
					order = 130,
				},
				lock = {
					name = L["Lock"],
					type = "toggle",
					desc = L["Toggle locking of the Custom Message Frame."],
					get = function(info) return PallyPowerWarn.db.profile.display.lock end,
					set = function(info,v)
						PallyPowerWarn.db.profile.display.lock = v
						PallyPowerWarn:UpdateLock()
					end,
					order = 131,
				},
				fontSize = {
					name = L["Font Size"],
					type = "range",
					desc = L["Set the font size in the Custom Message Frame."],
					get = function(info) return PallyPowerWarn.db.profile.display.fontSize end,
					set = function(info,v)
						PallyPowerWarn.db.profile.display.fontSize = v
						PallyPowerWarn:UpdateFont()
					end,
					step = 1,
					min = 8,
					max = 32,
					order = 132,
				},
				fontFace = {
					name = L["Font Face"],
					type = "select",
					desc = L["Set the font face in the Custom Message Frame."],
					get = function(info) return PallyPowerWarn.db.profile.display.fontFace end,
					set = function(info,v)
						PallyPowerWarn.db.profile.display.fontFace = v
						PallyPowerWarn:UpdateFont()
					end,
					values = {
						[L["FRIZQT__.TTF"]] = L["Normal"],
						[L["ARIALN.TTF"]] = L["Arial"],
						[L["skurri.ttf"]] = L["Skurri"],
						[L["MORPHEUS.ttf"]] = L["Morpheus"],
					},
					order = 133,
				},
				fontEffect = {
					name = L["Font Effect"],
					type = "select",
					desc = L["Set the font effect in the Custom Message Frame."],
					get = function(info) return PallyPowerWarn.db.profile.display.fontEffect end,
					set = function(info,v)
						PallyPowerWarn.db.profile.display.fontEffect = v
						PallyPowerWarn:UpdateFont()
					end,
					values = {
						["none"] = L["None"],
						[L["OUTLINE"]] = L["OUTLINE"],
						[L["THICKOUTLINE"]] = L["THICKOUTLINE"],
						[L["MONOCHROME"]] = L["MONOCHROME"],
					},
					order = 134,
				},
				soundheader = {
					name = L["Sound Channel"],
					type = "header",
					order = 135,
				},
				SoundChannel = {
					name = L["Sound Channel"],
					type = "select",
					desc = L["Channel to play sounds"],
					get = function(info) return PallyPowerWarn.db.profile.soundchannel end,
					set = function(info,v)
						PallyPowerWarn.db.profile.soundchannel = v
					end,
					values = {
						["SFX"] = L["Effects"],
						["Music"] = L["Music"],
						["Ambience"] = L["Ambience"],
						["Master"] = L["Master"],
					},
					order = 136,
				},
			}
		},
		blessing = {
			name = L["Blessings"],
			type = 'group',
			desc = L["Alerts for blessings."],
			order = 200, 
			args = {
				info = {
					type = "description",
					order = 201,
					name = L["Alerts for blessings."],
				},
				enable = {
					name = L["Enable"],
					type = "toggle",
					desc = L["Enable checking blessings."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.enable end,
					set = function(info,v)
						PallyPowerWarn.db.profile.blessing.enable = v
						PallyPowerWarn:LoadEvents()
					end,
					order = 202, 
				},
				whenheader = {
					order = 203,
					type = "header",
					name = L["When to check"],
				},
				readycheck = {
					name = L["Ready Check"],
					type = "toggle",
					desc = L["Notify on ready check."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.readycheck end,
					set = function(info,v)
						PallyPowerWarn.db.profile.blessing.readycheck = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 204, 
				},
				entercombat = {
					name = L["Enter Combat"],
					type = "toggle",
					desc = L["Notify when entering combat."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.entercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.blessing.entercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 205,
				},
				aftercombat = {
					name = L["After Combat"],
					type = "toggle",
					desc = L["Notify after the end of combat."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.aftercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.blessing.aftercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 206,
				},
				disableheader = {
					order = 207,
					type = "header",
					name = L["Disable checks"],
				},
				mounted = {
					name = L["No Mounted"],
					type = "toggle",
					desc = L["Disable notifications when mounted."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.mounted end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.mounted = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 208,
				},
				vehicle = {
					name = L["No Vehicle"],
					type = "toggle",
					desc = L["Disable notifications when in a vehicle."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.vehicle end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.vehicle = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 209,
				},
				combat = {
					name = L["No Combat"],
					type = "toggle",
					desc = L["Disable notifications when in combat."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.combat end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.combat = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 210,
				},
				pvp = {
					name = L["No PvP"],
					type = "toggle",
					desc = L["Disable notifications when PvP flagged."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.pvp end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.pvp = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 211,
				},
				locationheader = {
					order = 212,
					type = "header",
					name = L["Location"],
				},
				bg = {
					name = L["Battleground"],
					type = "toggle",
					desc = L["Warn when in battlegrounds."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.bg end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.bg = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 213,
				},
				arena = {
					name = L["Arena"],
					type = "toggle",
					desc = L["Warn when in arena."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.arena end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.arena = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 214,
				},
				wintergrasp = {
					name = L["ZONE_TOLB"],
					type = "toggle",
					desc = L["Warn when in Tol Barad/Wintergrasp."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.wg end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.wg = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 215,
				},
				instance = {
					name = L["5-man"],
					type = "toggle",
					desc = L["Warn when in a 5-man instance."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.instance end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.instance = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 216,
				},
				raid = {
					name = L["Raid"],
					type = "toggle",
					desc = L["Warn when in a raid instance."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.raid end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.raid = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 217,
				},
				world = {
					name = L["Other"],
					type = "toggle",
					desc = L["Warn when not in an instance, arena, or battleground."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.world end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.world = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 218,
				},
				wrongheader = {
					order = 219,
					type = "header",
					name = L["Wrong Buffs"],
				},
				wrong = {
					name = L["Wrong Buffs"],
					type = "toggle",
					desc = L["Warn for wrong buffs."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.wrong end,
					set = function(info,v) PallyPowerWarn.db.profile.blessing.wrong = v end,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 220,
				},
				wronginfo = {
					type = "description",
					order = 221,
					name = "Displays a warning if your blessings do not match PallyPower.",
				},
				otherheader = {
					order = 222,
					type = "header",
					name = L["Other Options"],
				},
				time = {
					name = L["Frequency"],
					type = "range",
					desc = L["Do not warn more often than (5=default)"],
					get = function(info) return PallyPowerWarn.db.profile.blessing.frequency end,
					set = function(info,v)
						PallyPowerWarn.db.profile.blessing.frequency = v
						end,
					step = 1,
					min = 0,
					max = 30,
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 223,
				},
				sound = {
					name = L["Sound"],
					type = "select",
					desc = L["Play a sound when a buff is missing."],
					get = function(info) return PallyPowerWarn.db.profile.blessing.sound end,
					set = function(info,v)
						PallyPowerWarn.db.profile.blessing.sound = v
						PallyPowerWarn:Play(v)
						end,
					values = {
						["ding"] = L["Ding"],
						["dong"] = L["Dong"],
						["dodo"] = L["Dodo"],
						["bell"] = L["Bell"],
						["none"] = L["None"],
					},
					disabled = function() return not PallyPowerWarn.db.profile.blessing.enable end,
					order = 224,
				},
				display = {
					name = L["Display"],
					type = 'group',
					desc = L["Settings for how to display the message."], 
					args = {
						color = {
							name = L["Color"],
							type = 'color',
							desc = L["Sets the color of the text when displaying messages."],
							get = function(info)
							local v = PallyPowerWarn.db.profile.blessing.display.color
								return v.r,v.g,v.b
								end,
							set = function(info,r,g,b) PallyPowerWarn.db.profile.blessing.display.color = {r=r,g=g,b=b} end
						},
						scroll = {
							type = 'toggle',
							name = L["Scroll output"],
							desc = L["Toggle showing messages in scrolling text. (Settings are in the 'Output Category')"],
							get = function(info) return PallyPowerWarn.db.profile.blessing.display.scroll end,
							set = function(info,t) PallyPowerWarn.db.profile.blessing.display.scroll = t end,
						},
						frames = {
							type = 'toggle',
							name = L["Frames output"],
							desc = L["Toggle whether the message will show up in the frames. (Settings are in the 'General Display' category.)"],
							get = function(info) return PallyPowerWarn.db.profile.blessing.display.frames end,
							set = function(info,t) PallyPowerWarn.db.profile.blessing.display.frames = t end,
						},
						time = {
							--hidden = true,
							name = L["Time to display message"],
							type = "range",
							desc = L["Set the time the message will be displayed (5=default)"],
							get = function(info) return PallyPowerWarn.db.profile.blessing.display.time end,
							set = function(info,v)
								PallyPowerWarn.db.profile.blessing.display.time = v
							end,
							step = 1,
							min = 1,
							max = 20,
							order = 111,
						},
					}
				},
			}
		},
		seal = {
			name = L["Seals"],
			type = 'group',
			desc = L["Alerts for seals."],
			order = 300, 
			args = {
				info = {
					type = "description",
					order = 301,
					name = L["Alerts for seals."],
				},
				enable = {
					name = L["Enable"],
					type = "toggle",
					desc = L["Enable checking seals."],
					get = function(info) return PallyPowerWarn.db.profile.seal.enable end,
					set = function(info,v)
						PallyPowerWarn.db.profile.seal.enable = v
						PallyPowerWarn:LoadEvents()
					end,
					order = 302, 
				},
				whenheader = {
					order = 303,
					type = "header",
					name = L["When to check"],
				},
				readycheck = {
					name = L["Ready Check"],
					type = "toggle",
					desc = L["Notify on ready check."],
					get = function(info) return PallyPowerWarn.db.profile.seal.readycheck end,
					set = function(info,v)
						PallyPowerWarn.db.profile.seal.readycheck = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.seal.enable end,
					order = 304, 
				},
				entercombat = {
					name = L["Enter Combat"],
					type = "toggle",
					desc = L["Notify when entering combat."],
					get = function(info) return PallyPowerWarn.db.profile.seal.entercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.seal.entercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.seal.enable end,
					order = 305,
				},
				aftercombat = {
					name = L["After Combat"],
					type = "toggle",
					desc = L["Notify after the end of combat."],
					get = function(info) return PallyPowerWarn.db.profile.seal.aftercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.seal.aftercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.seal.enable end,
					order = 306,
				},
				disableheader = {
					order = 307,
					type = "header",
					name = L["Disable checks"],
				},
				mounted = {
					name = L["No Mounted"],
					type = "toggle",
					desc = L["Disable notifications when mounted."],
					get = function(info) return PallyPowerWarn.db.profile.seal.mounted end,
					set = function(info,v) PallyPowerWarn.db.profile.seal.mounted = v end,
					disabled = function() return not PallyPowerWarn.db.profile.seal.enable end,
					order = 308,
				},
				vehicle = {
					name = L["No Vehicle"],
					type = "toggle",
					desc = L["Disable notifications when in a vehicle."],
					get = function(info) return PallyPowerWarn.db.profile.seal.vehicle end,
					set = function(info,v) PallyPowerWarn.db.profile.seal.vehicle = v end,
					disabled = function() return not PallyPowerWarn.db.profile.seal.enable end,
					order = 309,
				},
				combat = {
					name = L["No Combat"],
					type = "toggle",
					desc = L["Disable notifications when in combat."],
					get = function(info) return PallyPowerWarn.db.profile.seal.combat end,
					set = function(info,v) PallyPowerWarn.db.profile.seal.combat = v end,
					disabled = function() return not PallyPowerWarn.db.profile.seal.enable end,
					order = 310,
				},
				wrongheader = {
					order = 311,
					type = "header",
					name = L["Wrong Seal"],
				},
				wrong = {
					name = L["Wrong Seal"],
					type = "toggle",
					desc = L["Warn for wrong seal."],
					get = function(info) return PallyPowerWarn.db.profile.seal.wrong end,
					set = function(info,v) PallyPowerWarn.db.profile.seal.wrong = v end,
					disabled = function() return not PallyPowerWarn.db.profile.seal.enable end,
					order = 312,
				},
				wronginfo = {
					type = "description",
					order = 313,
					name = "Displays a warning if your seal does not match PallyPower. If you are not using PallyPower it checks seal by spec.",
				},
				otherheader = {
					order = 314,
					type = "header",
					name = L["Other Options"],
				},
				time = {
					name = L["Frequency"],
					type = "range",
					desc = L["Do not warn more often than (5=default)"],
					get = function(info) return PallyPowerWarn.db.profile.seal.frequency end,
					set = function(info,v)
						PallyPowerWarn.db.profile.seal.frequency = v
						end,
					step = 1,
					min = 0,
					max = 30,
					disabled = function() return not PallyPowerWarn.db.profile.seal.enable end,
					order = 315,
				},
				sound = {
					name = L["Sound"],
					type = "select",
					desc = L["Play a sound when a buff is missing."],
					get = function(info) return PallyPowerWarn.db.profile.seal.sound end,
					set = function(info,v)
						PallyPowerWarn.db.profile.seal.sound = v
						PallyPowerWarn:Play(v)
						end,
					values = {
						["ding"] = L["Ding"],
						["dong"] = L["Dong"],
						["dodo"] = L["Dodo"],
						["bell"] = L["Bell"],
						["none"] = L["None"],
					},
					disabled = function() return not PallyPowerWarn.db.profile.seal.enable end,
					order = 316,
				},
				display = {
					name = L["Display"],
					type = 'group',
					desc = L["Settings for how to display the message."], 
					args = {
						color = {
							name = L["Color"],
							type = 'color',
							desc = L["Sets the color of the text when displaying messages."],
							get = function(info)
							local v = PallyPowerWarn.db.profile.seal.display.color
								return v.r,v.g,v.b
								end,
							set = function(info,r,g,b) PallyPowerWarn.db.profile.seal.display.color = {r=r,g=g,b=b} end
						},
						scroll = {
							type = 'toggle',
							name = L["Scroll output"],
							desc = L["Toggle showing messages in scrolling text. (Settings are in the 'Output Category')"],
							get = function(info) return PallyPowerWarn.db.profile.seal.display.scroll end,
							set = function(info,t) PallyPowerWarn.db.profile.seal.display.scroll = t end,
						},
						frames = {
							type = 'toggle',
							name = L["Frames output"],
							desc = L["Toggle whether the message will show up in the frames. (Settings are in the 'General Display' category.)"],
							get = function(info) return PallyPowerWarn.db.profile.seal.display.frames end,
							set = function(info,t) PallyPowerWarn.db.profile.seal.display.frames = t end,
						},
						time = {
							--hidden = true,
							name = L["Time to display message"],
							type = "range",
							desc = L["Set the time the message will be displayed (5=default)"],
							get = function(info) return PallyPowerWarn.db.profile.seal.display.time end,
							set = function(info,v)
								PallyPowerWarn.db.profile.seal.display.time = v
							end,
							step = 1,
							min = 1,
							max = 20,
							order = 111,
						},
					}
				},
			}
		},
		rf = {
			name = L["Righteous Fury"],
			type = 'group',
			desc = L["Alerts for Righteous Fury."],
			order = 400, 
			args = {
				info = {
					type = "description",
					order = 401,
					name = L["Alerts for Righteous Fury."],
				},
				enable = {
					name = L["Enable"],
					type = "toggle",
					desc = L["Enable checking Righteous Fury."],
					get = function(info) return PallyPowerWarn.db.profile.rf.enable end,
					set = function(info,v)
						PallyPowerWarn.db.profile.rf.enable = v
						PallyPowerWarn:LoadEvents()
					end,
					order = 402, 
				},
				tankspec = {
					name = L["Spec Check"],
					type = "select",
					desc = L["Spec Check"],
					get = function(info) return PallyPowerWarn.db.profile.rf.tankspec end,
					set = function(info,v)
						PallyPowerWarn.db.profile.rf.tankspec = v
					end,
					values = {
						["both"] = L["Both specs"],
						["prot"] = L["Only in Prot spec"],
						["primary"] = L["Only in primary spec"],
						["secondary"] = L["Only in secondary spec"],
					},
					disabled = function() return not PallyPowerWarn.db.profile.rf.enable end,
					order = 403,
				},
				whenheader = {
					order = 404,
					type = "header",
					name = L["When to check"],
				},
				readycheck = {
					name = L["Ready Check"],
					type = "toggle",
					desc = L["Notify on ready check."],
					get = function(info) return PallyPowerWarn.db.profile.rf.readycheck end,
					set = function(info,v)
						PallyPowerWarn.db.profile.rf.readycheck = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.rf.enable end,
					order = 405, 
				},
				entercombat = {
					name = L["Enter Combat"],
					type = "toggle",
					desc = L["Notify when entering combat."],
					get = function(info) return PallyPowerWarn.db.profile.rf.entercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.rf.entercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.rf.enable end,
					order = 406,
				},
				aftercombat = {
					name = L["After Combat"],
					type = "toggle",
					desc = L["Notify after the end of combat."],
					get = function(info) return PallyPowerWarn.db.profile.rf.aftercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.rf.aftercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.rf.enable end,
					order = 407,
				},
				disableheader = {
					order = 408,
					type = "header",
					name = L["Disable checks"],
				},
				mounted = {
					name = L["No Mounted"],
					type = "toggle",
					desc = L["Disable notifications when mounted."],
					get = function(info) return PallyPowerWarn.db.profile.rf.mounted end,
					set = function(info,v) PallyPowerWarn.db.profile.rf.mounted = v end,
					disabled = function() return not PallyPowerWarn.db.profile.rf.enable end,
					order = 409,
				},
				vehicle = {
					name = L["No Vehicle"],
					type = "toggle",
					desc = L["Disable notifications when in a vehicle."],
					get = function(info) return PallyPowerWarn.db.profile.rf.vehicle end,
					set = function(info,v) PallyPowerWarn.db.profile.rf.vehicle = v end,
					disabled = function() return not PallyPowerWarn.db.profile.rf.enable end,
					order = 410,
				},
				otherheader = {
					order = 411,
					type = "header",
					name = L["Other Options"],
				},
				time = {
					name = L["Frequency"],
					type = "range",
					desc = L["Do not warn more often than (5=default)"],
					get = function(info) return PallyPowerWarn.db.profile.rf.frequency end,
					set = function(info,v)
						PallyPowerWarn.db.profile.rf.frequency = v
						end,
					step = 1,
					min = 0,
					max = 30,
					disabled = function() return not PallyPowerWarn.db.profile.rf.enable end,
					order = 412,
				},
				sound = {
					name = L["Sound"],
					type = "select",
					desc = L["Play a sound when a buff is missing."],
					get = function(info) return PallyPowerWarn.db.profile.rf.sound end,
					set = function(info,v)
						PallyPowerWarn.db.profile.rf.sound = v
						PallyPowerWarn:Play(v)
						end,
					values = {
						["ding"] = L["Ding"],
						["dong"] = L["Dong"],
						["dodo"] = L["Dodo"],
						["bell"] = L["Bell"],
						["none"] = L["None"],
					},
					disabled = function() return not PallyPowerWarn.db.profile.rf.enable end,
					order = 413,
				},
				display = {
					name = L["Display"],
					type = 'group',
					desc = L["Settings for how to display the message."], 
					args = {
						color = {
							name = L["Color"],
							type = 'color',
							desc = L["Sets the color of the text when displaying messages."],
							get = function(info)
							local v = PallyPowerWarn.db.profile.rf.display.color
								return v.r,v.g,v.b
								end,
							set = function(info,r,g,b) PallyPowerWarn.db.profile.rf.display.color = {r=r,g=g,b=b} end
						},
						scroll = {
							type = 'toggle',
							name = L["Scroll output"],
							desc = L["Toggle showing messages in scrolling text. (Settings are in the 'Output Category')"],
							get = function(info) return PallyPowerWarn.db.profile.rf.display.scroll end,
							set = function(info,t) PallyPowerWarn.db.profile.rf.display.scroll = t end,
						},
						frames = {
							type = 'toggle',
							name = L["Frames output"],
							desc = L["Toggle whether the message will show up in the frames. (Settings are in the 'General Display' category.)"],
							get = function(info) return PallyPowerWarn.db.profile.rf.display.frames end,
							set = function(info,t) PallyPowerWarn.db.profile.rf.display.frames = t end,
						},
						time = {
							--hidden = true,
							name = L["Time to display message"],
							type = "range",
							desc = L["Set the time the message will be displayed (5=default)"],
							get = function(info) return PallyPowerWarn.db.profile.rf.display.time end,
							set = function(info,v)
								PallyPowerWarn.db.profile.rf.display.time = v
							end,
							step = 1,
							min = 1,
							max = 20,
							order = 111,
						},
					}
				},
			}
		},
		aura = {
			name = L["Auras"],
			type = 'group',
			desc = L["Alerts for auras."],
			order = 500, 
			args = {
				info = {
					type = "description",
					order = 501,
					name = L["Alerts for auras."],
				},
				enable = {
					name = L["Enable"],
					type = "toggle",
					desc = L["Enable checking auras."],
					get = function(info) return PallyPowerWarn.db.profile.aura.enable end,
					set = function(info,v)
						PallyPowerWarn.db.profile.aura.enable = v
						PallyPowerWarn:LoadEvents()
					end,
					order = 502, 
				},
				whenheader = {
					order = 503,
					type = "header",
					name = L["When to check"],
				},
				readycheck = {
					name = L["Ready Check"],
					type = "toggle",
					desc = L["Notify on ready check."],
					get = function(info) return PallyPowerWarn.db.profile.aura.readycheck end,
					set = function(info,v)
						PallyPowerWarn.db.profile.aura.readycheck = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 504, 
				},
				entercombat = {
					name = L["Enter Combat"],
					type = "toggle",
					desc = L["Notify when entering combat."],
					get = function(info) return PallyPowerWarn.db.profile.aura.entercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.aura.entercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 505,
				},
				aftercombat = {
					name = L["After Combat"],
					type = "toggle",
					desc = L["Notify after the end of combat."],
					get = function(info) return PallyPowerWarn.db.profile.aura.aftercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.aura.aftercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 506,
				},
				disableheader = {
					order = 507,
					type = "header",
					name = L["Disable checks"],
				},
				mounted = {
					name = L["No Mounted"],
					type = "toggle",
					desc = L["Disable notifications when mounted."],
					get = function(info) return PallyPowerWarn.db.profile.aura.mounted end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.mounted = v end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 508,
				},
				vehicle = {
					name = L["No Vehicle"],
					type = "toggle",
					desc = L["Disable notifications when in a vehicle."],
					get = function(info) return PallyPowerWarn.db.profile.aura.vehicle end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.vehicle = v end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 509,
				},
				wrongheader = {
					order = 510,
					type = "header",
					name = L["Wrong Aura"],
				},
				overwritten = {
					name = L["Aura Overwritten"],
					type = "toggle",
					desc = L["Warns if your aura is being overwritten by a more powerful version from another paladin."],
					get = function(info) return PallyPowerWarn.db.profile.aura.overwritten end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.overwritten = v end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 511,
				},
				wrong = {
					name = L["Wrong Aura"],
					type = "select",
					-- style = "radio",
					desc = L["Warn for wrong aura."],
					get = function(info) return PallyPowerWarn.db.profile.aura.wrong end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.wrong = v end,
					values = {
						["any"] = L["Any aura is fine"],
						["crusader"] = L["Warn for Crusader only"],
						["pp"] = L["Non-PallyPower Aura"],
					},
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 512,
				},
				wronginfo = {
					type = "description",
					order = 513,
					name = "Options:\nAny aura is fine - No warnings for wrong aura.\n\nWarn for Crusader only - Will warn if you have crusader and PallyPower is not set to Crusader.\n\nNon-PallyPower Aura - Will warn if your aura is different than PallyPower.",
				},
				bg = {
					name = L["Battleground"],
					type = "toggle",
					desc = L["Warn when in battlegrounds."],
					get = function(info) return PallyPowerWarn.db.profile.aura.bg end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.bg = v end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 514,
				},
				arena = {
					name = L["Arena"],
					type = "toggle",
					desc = L["Warn when in arena."],
					get = function(info) return PallyPowerWarn.db.profile.aura.arena end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.arena = v end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 515,
				},
				wintergrasp = {
					name = L["ZONE_TOLB"],
					type = "toggle",
					desc = L["Warn when in Tol Barad/Wintergrasp."],
					get = function(info) return PallyPowerWarn.db.profile.aura.wg end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.wg = v end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 516,
				},
				instance = {
					name = L["5-man"],
					type = "toggle",
					desc = L["Warn when in a 5-man instance."],
					get = function(info) return PallyPowerWarn.db.profile.aura.instance end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.instance = v end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 517,
				},
				raid = {
					name = L["Raid"],
					type = "toggle",
					desc = L["Warn when in a raid instance."],
					get = function(info) return PallyPowerWarn.db.profile.aura.raid end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.raid = v end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 518,
				},
				world = {
					name = L["Other"],
					type = "toggle",
					desc = L["Warn when not in an instance, arena, or battleground."],
					get = function(info) return PallyPowerWarn.db.profile.aura.world end,
					set = function(info,v) PallyPowerWarn.db.profile.aura.world = v end,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 519,
				},
				otherheader = {
					order = 521,
					type = "header",
					name = L["Other Options"],
				},
				time = {
					name = L["Frequency"],
					type = "range",
					desc = L["Do not warn more often than (5=default)"],
					get = function(info) return PallyPowerWarn.db.profile.aura.frequency end,
					set = function(info,v)
						PallyPowerWarn.db.profile.aura.frequency = v
						end,
					step = 1,
					min = 0,
					max = 30,
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 522,
				},
				sound = {
					name = L["Sound"],
					type = "select",
					desc = L["Play a sound when a buff is missing."],
					get = function(info) return PallyPowerWarn.db.profile.aura.sound end,
					set = function(info,v)
						PallyPowerWarn.db.profile.aura.sound = v
						PallyPowerWarn:Play(v)
						end,
					values = {
						["ding"] = L["Ding"],
						["dong"] = L["Dong"],
						["dodo"] = L["Dodo"],
						["bell"] = L["Bell"],
						["none"] = L["None"],
					},
					disabled = function() return not PallyPowerWarn.db.profile.aura.enable end,
					order = 523,
				},
				display = {
					name = L["Display"],
					type = 'group',
					desc = L["Settings for how to display the message."], 
					args = {
						color = {
							name = L["Color"],
							type = 'color',
							desc = L["Sets the color of the text when displaying messages."],
							get = function(info)
							local v = PallyPowerWarn.db.profile.aura.display.color
								return v.r,v.g,v.b
								end,
							set = function(info,r,g,b) PallyPowerWarn.db.profile.aura.display.color = {r=r,g=g,b=b} end
						},
						scroll = {
							type = 'toggle',
							name = L["Scroll output"],
							desc = L["Toggle showing messages in scrolling text. (Settings are in the 'Output Category')"],
							get = function(info) return PallyPowerWarn.db.profile.aura.display.scroll end,
							set = function(info,t) PallyPowerWarn.db.profile.aura.display.scroll = t end,
						},
						frames = {
							type = 'toggle',
							name = L["Frames output"],
							desc = L["Toggle whether the message will show up in the frames. (Settings are in the 'General Display' category.)"],
							get = function(info) return PallyPowerWarn.db.profile.aura.display.frames end,
							set = function(info,t) PallyPowerWarn.db.profile.aura.display.frames = t end,
						},
						time = {
							--hidden = true,
							name = L["Time to display message"],
							type = "range",
							desc = L["Set the time the message will be displayed (5=default)"],
							get = function(info) return PallyPowerWarn.db.profile.aura.display.time end,
							set = function(info,v)
								PallyPowerWarn.db.profile.aura.display.time = v
							end,
							step = 1,
							min = 1,
							max = 20,
							order = 111,
						},
					}
				},
			}
		},
		announce = {
			name = L["Announcements"],
			type = 'group',
			childGroups = 'tab',
			desc = L["Announce different spells."],
			order = 600,
			args = {
				spells = {
					name = L["Spells"],
					type = 'group',
					desc = L["Spells to announce."], 
					order = 100,
					args = {
						intheader = {
							order = 101,
							type = "header",
							name = L["Interrupt"],
						},
						interrupt = {
							name = L["Interrupt"],
							type = "toggle",
							desc = L["Toggle Interrupt information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.interrupt end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.interrupt = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 102,
						},
						intchan = {
							name = L["Announce Interrupt"],
							type = "select",
							desc = L["Announce Interrupt message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.intchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.intchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.interrupt end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 103,
						},
						dispelheader = {
							order = 104,
							type = "header",
							name = L["Dispel"],
						},
						dispel = {
							name = L["Dispel"],
							type = "toggle",
							desc = L["Toggle message when dispel is cast."],
							get = function(info) return PallyPowerWarn.db.profile.announce.dispel end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.dispel = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 105,
						},
						dispelpm = {
							name = L["Whisper"],
							type = "toggle",
							desc = L["Toggle dispel whisper."],
							get = function(info) return PallyPowerWarn.db.profile.announce.dispelpm end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.dispelpm = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.dispel end,
							order = 106,
						},
						dispelchan = {
							name = L["Announce Dispel"],
							type = "select",
							desc = L["Announce Dispel message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.dispelchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.dispelchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.dispel end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 107,
						},
						hofheader = {
							order = 108,
							type = "header",
							name = L["Hand of Freedom"],
						},
						hof = {
							name = L["Hand of Freedom"],
							type = "toggle",
							desc = L["Toggle message when HoF is cast."],
							get = function(info) return PallyPowerWarn.db.profile.announce.hof end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.hof = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 109,
						},
						hofpm = {
							name = L["Whisper"],
							type = "toggle",
							desc = L["Toggle HoF whisper."],
							get = function(info) return PallyPowerWarn.db.profile.announce.hofpm end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.hofpm = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.hof end,
							order = 110,
						},
						hofchan = {
							name = L["Announce HoF"],
							type = "select",
							desc = L["Announce HoF message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.hofchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.hofchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.hof end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 111,
						},
						hoftext = {
							name = L["HoF text"],
							type = "input",
							width = "full",
							desc = L["HoF announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.hoftext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.hoftext = v
							end,
							usage = L["SPELL = spell cast, TARGET = target of spell"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.hof end,
							order = 112,
						},
						hopheader = {
							order = 113,
							type = "header",
							name = L["Hand of Protection"],
						},
						hop = {
							name = L["Hand of Protection"],
							type = "toggle",
							desc = L["Toggle message when HoP is cast."],
							get = function(info) return PallyPowerWarn.db.profile.announce.hop end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.hop = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 114,
						},
						hoppm = {
							name = L["Whisper"],
							type = "toggle",
							desc = L["Toggle HoP whisper."],
							get = function(info) return PallyPowerWarn.db.profile.announce.hoppm end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.hoppm = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.hop end,
							order = 115,
						},
						hopchan = {
							name = L["Announce HoP"],
							type = "select",
							desc = L["Announce HoP message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.hopchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.hopchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.hop end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 116,
						},
						hoptext = {
							name = L["HoP text"],
							type = "input",
							width = "full",
							desc = L["HoP announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.hoptext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.hoptext = v
							end,
							usage = L["SPELL = spell cast, TARGET = target of spell"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.hop end,
							order = 117,
						},
						sacheader = {
							order = 118,
							type = "header",
							name = L["Hand of Sacrifice"],
						},
						sac = {
							name = L["Hand of Sacrifice"],
							type = "toggle",
							desc = L["Toggle message when Sacrifice is cast."],
							get = function(info) return PallyPowerWarn.db.profile.announce.sac end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.sac = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 119,
						},
						sacpm = {
							name = L["Whisper"],
							type = "toggle",
							desc = L["Toggle Sacrifice whisper."],
							get = function(info) return PallyPowerWarn.db.profile.announce.sacpm end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.sacpm = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.sac end,
							order = 120,
						},
						sacchan = {
							name = L["Announce Sacrifice"],
							type = "select",
							desc = L["Announce Sacrifice message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.sacchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.sacchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.sac end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 121,
						},
						sactext = {
							name = L["Sacrifice text"],
							type = "input",
							width = "full",
							desc = L["Sacrifice announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.sactext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.sactext = v
							end,
							usage = L["SPELL = spell cast, TARGET = target of spell"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.sac end,
							order = 122,
						},
						salvheader = {
							order = 123,
							type = "header",
							name = L["Hand of Salvation"],
						},
						salv = {
							name = L["Hand of Salvation"],
							type = "toggle",
							desc = L["Toggle message when Salvation is cast."],
							get = function(info) return PallyPowerWarn.db.profile.announce.salv end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.salv = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 124,
						},
						salvpm = {
							name = L["Whisper"],
							type = "toggle",
							desc = L["Toggle Salvation whisper."],
							get = function(info) return PallyPowerWarn.db.profile.announce.salvpm end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.salvpm = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.salv end,
							order = 125,
						},
						salvchan = {
							name = L["Announce Salvation"],
							type = "select",
							desc = L["Announce Salvation message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.salvchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.salvchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.salv end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 126,
						},
						salvtext = {
							name = L["Salvation text"],
							type = "input",
							width = "full",
							desc = L["Salvation announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.salvtext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.salvtext = v
							end,
							usage = L["SPELL = spell cast, TARGET = target of spell"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.salv end,
							order = 127,
						},
						avengheader = {
							order = 128,
							type = "header",
							name = L["Avenging Wrath"],
						},
						aveng = {
							name = L["Avenging Wrath"],
							type = "toggle",
							desc = L["Toggle Avenging Wrath information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.aveng end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.aveng = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 129,
						},
						avengchan = {
							name = L["Announce Avenging Wrath"],
							type = "select",
							desc = L["Announce Avenging Wrath message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.avengchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.avengchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.aveng end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 130,
						},
						avengtext = {
							name = L["Avenging Wrath text"],
							type = "input",
							width = "full",
							desc = L["Avenging Wrath announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.avengtext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.avengtext = v
							end,
							usage = L["SPELL = spell cast"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.aveng end,
							order = 131,
						},
						pleaheader = {
							order = 132,
							type = "header",
							name = L["Divine Plea"],
						},
						plea = {
							name = L["Divine Plea"],
							type = "toggle",
							desc = L["Toggle Divine Plea information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.plea end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.plea = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 133,
						},
						pleachan = {
							name = L["Announce Divine Plea"],
							type = "select",
							desc = L["Announce Divine Plea message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.pleachan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.pleachan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.plea end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 134,
						},
						pleatext = {
							name = L["Divine Plea text"],
							type = "input",
							width = "full",
							desc = L["Divine Plea announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.pleatext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.pleatext = v
							end,
							usage = L["SPELL = spell cast"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.plea end,
							order = 135,
						},
						zealheader = {
							order = 136,
							type = "header",
							name = L["Zealotry"],
						},
						zeal = {
							name = L["Zealotry"],
							type = "toggle",
							desc = L["Toggle Zealotry information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.zeal end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.zeal = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 137,
						},
						zealchan = {
							name = L["Announce Zealotry"],
							type = "select",
							desc = L["Announce Zealotry message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.zealchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.zealchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.zeal end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 138,
						},
						zealtext = {
							name = L["Zealotry text"],
							type = "input",
							width = "full",
							desc = L["Zealotry announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.zealtext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.zealtext = v
							end,
							usage = L["SPELL = spell cast"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.zeal end,
							order = 139,
						},
						shieldheader = {
							order = 140,
							type = "header",
							name = L["Divine Shield"],
						},
						shield = {
							name = L["Divine Shield"],
							type = "toggle",
							desc = L["Toggle Divine Shield information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.shield end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.shield = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 141,
						},
						shieldchan = {
							name = L["Announce Divine Shield"],
							type = "select",
							desc = L["Announce Divine Shield message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.shieldchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.shieldchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.shield end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 142,
						},
						shieldtext = {
							name = L["Divine Shield text"],
							type = "input",
							width = "full",
							desc = L["Divine Shield announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.shieldtext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.shieldtext = v
							end,
							usage = L["SPELL = spell cast"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.shield end,
							order = 143,
						},
						amheader = {
							order = 144,
							type = "header",
							name = L["Aura Mastery"],
						},
						am = {
							name = L["Aura Mastery"],
							type = "toggle",
							desc = L["Toggle Aura Mastery information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.am end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.am = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 145,
						},
						amchan = {
							name = L["Announce Aura Mastery"],
							type = "select",
							desc = L["Announce Aura Mastery message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.amchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.amchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.am end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 146,
						},
						amtext = {
							name = L["Aura Mastery text"],
							type = "input",
							width = "full",
							desc = L["Aura Mastery announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.amtext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.amtext = v
							end,
							usage = L["SPELL = spell cast, AURA = Paladin aura"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.am end,
							order = 147,
						},
						lohheader = {
							order = 148,
							type = "header",
							name = L["Lay on Hands"],
						},
						loh = {
							name = L["Lay on Hands"],
							type = "toggle",
							desc = L["Toggle message when Lay on Hands is cast."],
							get = function(info) return PallyPowerWarn.db.profile.announce.loh end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.loh = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 149,
						},
						lohpm = {
							name = L["Whisper"],
							type = "toggle",
							desc = L["Toggle Lay on Hands whisper."],
							get = function(info) return PallyPowerWarn.db.profile.announce.lohpm end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.lohpm = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.loh end,
							order = 150,
						},
						lohchan = {
							name = L["Announce LoH"],
							type = "select",
							desc = L["Announce Lay on Hands message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.lohchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.lohchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.loh end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 151,
						},
						lohtext = {
							name = L["Lay on Hands text"],
							type = "input",
							width = "full",
							desc = L["Lay on Hands announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.lohtext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.lohtext = v
							end,
							usage = L["SPELL = spell cast, TARGET = target of spell, AMOUNT = amount healed"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.loh end,
							order = 152,
						},
					}
				},
				tank = {
					name = L["Tank"],
					type = 'group',
					desc = L["Spells to announce."], 
					order = 200,
					args = {
						tankheader = {
							order = 201,
							type = "header",
							name = L["Tank"],
						},
						tankspec = {
							name = L["Spec Check"],
							type = "select",
							width = "double",
							desc = L["Announce tank cooldowns by spec."],
							get = function(info) return PallyPowerWarn.db.profile.announce.tankspec end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.tankspec = v
							end,
							values = {
								["both"] = L["Both specs"],
								["prot"] = L["Only in Prot spec"],
								["primary"] = L["Only in primary spec"],
								["secondary"] = L["Only in secondary spec"],
								["none"] = L["Disable tank cooldown announcements"],
							},
							order = 202,
						},
						dgheader = {
							order = 203,
							type = "header",
							name = L["Divine Guardian"],
						},
						dg = {
							name = L["Divine Guardian"],
							type = "toggle",
							desc = L["Toggle Divine Guardian information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.dg end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.dg = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 204,
						},
						dgchan = {
							name = L["Announce Divine Guardian"],
							type = "select",
							desc = L["Announce Divine Guardian message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.dgchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.dgchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.dg end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 205,
						},
						dgtext = {
							name = L["Divine Guardian text"],
							type = "input",
							width = "full",
							desc = L["Divine Guardian announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.dgtext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.dgtext = v
							end,
							usage = L["SPELL = spell cast"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.dg end,
							order = 206,
						},
						dpheader = {
							order = 207,
							type = "header",
							name = L["Divine Protection"],
						},
						dp = {
							name = L["Divine Protection"],
							type = "toggle",
							desc = L["Toggle Divine Protection information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.dp end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.dp = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 208,
						},
						dpchan = {
							name = L["Announce Divine Protection"],
							type = "select",
							desc = L["Announce Divine Protection message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.dpchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.dpchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.dp end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 209,
						},
						dptext = {
							name = L["Divine Protection text"],
							type = "input",
							width = "full",
							desc = L["Divine Protection announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.dptext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.dptext = v
							end,
							usage = L["SPELL = spell cast"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.dp end,
							order = 210,
						},
						ardentheader = {
							order = 211,
							type = "header",
							name = L["Ardent Defender"],
						},
						ardent = {
							name = L["Ardent Defender"],
							type = "toggle",
							desc = L["Toggle Ardent Defender information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.ardent end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.ardent = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 212,
						},
						adchan = {
							name = L["Announce Ardent Defender"],
							type = "select",
							desc = L["Announce Ardent Defender message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.adchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.adchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.ardent end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 213,
						},
						adtext = {
							name = L["Ardent Defender text"],
							type = "input",
							width = "full",
							desc = L["Ardent Defender announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.adtext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.adtext = v
							end,
							usage = L["SPELL = spell cast"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.ardent end,
							order = 214
						},
						adproc = {
							name = L["Ardent Defender Proc"],
							type = "toggle",
							desc = L["Toggle Ardent Defender heal proc information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.adproc end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.adproc = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 215,
						},
						adprocchan = {
							name = L["Announce Ardent Defender Proc"],
							type = "select",
							desc = L["Announce Ardent Defender Proc message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.adprocchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.adprocchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.adproc end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 216,
						},
						adproctext = {
							name = L["Ardent Defender Proc text"],
							type = "input",
							width = "full",
							desc = L["Ardent Defender Proc announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.adproctext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.adproctext = v
							end,
							usage = L["SPELL = spell cast, AMOUNT = amount healed"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.adproc end,
							order = 217
						},
						goakheader = {
							order = 218,
							type = "header",
							name = L["Guardian of Ancient Kings"],
						},
						goak = {
							name = L["Guardian of Ancient Kings"],
							type = "toggle",
							width = "double",
							desc = L["Toggle GoAK information."],
							get = function(info) return PallyPowerWarn.db.profile.announce.goak end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.goak = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 219,
						},
						goakchan = {
							name = L["Announce GoAK"],
							type = "select",
							desc = L["Announce GoAK message to the following channel. (Above option must be enabled)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.goakchan end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.goakchan = v
							end,
							disabled = function() return not PallyPowerWarn.db.profile.announce.goak end,
							values = {
								["RAID"] = L["Raid"],
								["PARTY"] = L["Party"],
								["RAID_WARNING"] = L["Raid Warning"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 220,
						},
						goaktext = {
							name = L["Guardian of Ancient Kings text"],
							type = "input",
							width = "full",
							desc = L["Guardian of Ancient Kings announcement text."],
							get = function(info) return PallyPowerWarn.db.profile.announce.goaktext end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.goaktext = v
							end,
							usage = L["SPELL = spell cast"],
							disabled = function() return not PallyPowerWarn.db.profile.announce.goak end,
							order = 221,
						},
					}
				},
				announceloc = {
					name = L["Where"],
					type = 'group',
					desc = L["Where to display the announcements."], 
					order = 300,
					args = {
						BGAnnounce = {
							name = L["BG Announce"],
							type = "select",
							width = "double",
							desc = L["Announce when in battlegrounds."],
							get = function(info) return PallyPowerWarn.db.profile.announce.loc.bgannounce end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.loc.bgannounce = v
								PallyPowerWarn:LoadEvents()
							end,
							values = {
								["BATTLEGROUND"] = L["Battleground"],
								["PARTY"] = L["Party"],
								["YELL"] = L["Yell"],
								["SAY"] = L["Say"],
								["none"] = L["None"],
							},
							order = 351,
						},
						ArenaAnnounce = {
							name = L["Arena Announce"],
							type = "toggle",
							width = "double",
							desc = L["Announce when in arena."],
							get = function(info) return PallyPowerWarn.db.profile.announce.loc.arenaannounce end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.loc.arenaannounce = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 352,
						},
						PartyAnnounce = {
							name = L["5-man Announce"],
							type = "toggle",
							width = "double",
							desc = L["Announce when in a 5-man instance."],
							get = function(info) return PallyPowerWarn.db.profile.announce.loc.partyannounce end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.loc.partyannounce = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 353,
						},
						RaidAnnounce = {
							name = L["Raid Announce"],
							type = "toggle",
							width = "double",
							desc = L["Announce when in a raid instance."],
							get = function(info) return PallyPowerWarn.db.profile.announce.loc.raidannounce end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.loc.raidannounce = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 354,
						},
						WorldAnnounce = {
							name = L["World Announce"],
							type = "toggle",
							width = "double",
							desc = L["Announce when not in instances."],
							get = function(info) return PallyPowerWarn.db.profile.announce.loc.worldannounce end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.loc.worldannounce = v
								PallyPowerWarn:LoadEvents()
							end,
							order = 355,
						},
					}
				},
				display = {
					name = L["Display"],
					type = 'group',
					desc = L["Settings for how to display the message."], 
					order = 400,
					args = {
						color = {
							name = L["Color"],
							type = 'color',
							width = "double",
							desc = L["Sets the color of the text when displaying messages."],
							get = function(info)
							local v = PallyPowerWarn.db.profile.announce.display.color
								return v.r,v.g,v.b
								end,
							set = function(info,r,g,b) PallyPowerWarn.db.profile.announce.display.color = {r=r,g=g,b=b} end
						},
						scroll = {
							type = 'toggle',
							width = "double",
							name = L["Scroll output"],
							desc = L["Toggle showing messages in scrolling text. (Settings are in the 'Output Category')"],
							get = function(info) return PallyPowerWarn.db.profile.announce.display.scroll end,
							set = function(info,t) PallyPowerWarn.db.profile.announce.display.scroll = t end,
						},
						frames = {
							type = 'toggle',
							width = "double",
							name = L["Frames output"],
							desc = L["Toggle whether the message will show up in the frames. (Settings are in the 'General Display' category.)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.display.frames end,
							set = function(info,t) PallyPowerWarn.db.profile.announce.display.frames = t end,
						},
						time = {
							--hidden = true,
							name = L["Time to display message"],
							type = "range",
							width = "double",
							desc = L["Set the time the message will be displayed (5=default)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.display.time end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.display.time = v
							end,
							step = 1,
							min = 1,
							max = 20,
						},
					}
				},
				other = {
					name = L["Other Options"],
					type = 'group',
					desc = L["Other Options"], 
					order = 500,
					args = {
						target = {
							name = L["Add target"],
							type = "toggle",
							width = "double",
							desc = L["Add the target to the end of the message when announcing."],
							get = function(info) return PallyPowerWarn.db.profile.announce.target end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.target = v
							end,
							order = 501,
						},
						frequency = {
							name = L["Frequency"],
							type = "range",
							width = "double",
							desc = L["Do not warn more often than (5=default)"],
							get = function(info) return PallyPowerWarn.db.profile.announce.frequency end,
							set = function(info,v)
								PallyPowerWarn.db.profile.announce.frequency = v
								end,
							step = 1,
							min = 0,
							max = 30,
							order = 502,
						},
					}
				},
			}
		},
		taunt = {
			name = L["Taunts"],
			type = 'group',
			desc = L["Taunt Tracker"],
			order = 700,
			args = {
				enable = {
					name = L["Enable"],
					type = "toggle",
					desc = L["Toggle Taunt tracker"],
					get = function(info) return PallyPowerWarn.db.profile.taunt.enable end,
					set = function(info,v)
						PallyPowerWarn.db.profile.taunt.enable = v
						PallyPowerWarn:LoadEvents()
					end,
					order = 701,
				},
				own = {
					name = L["Own"],
					type = "toggle",
					desc = L["Track own taunts"],
					get = function(info) return PallyPowerWarn.db.profile.taunt.own end,
					set = function(info,v)
						PallyPowerWarn.db.profile.taunt.own = v
					end,
					disabled = function() return not PallyPowerWarn.db.profile.taunt.enable end,
					order = 702,
				},
				other = {
					name = L["Other"],
					type = "toggle",
					desc = L["Track other player's taunts"],
					get = function(info) return PallyPowerWarn.db.profile.taunt.other end,
					set = function(info,v)
						PallyPowerWarn.db.profile.taunt.other = v
					end,
					disabled = function() return not PallyPowerWarn.db.profile.taunt.enable end,
					order = 703,
				},
				sound = {
					name = L["Sound"],
					type = "select",
					desc = L["Play a sound when someone taunts."],
					get = function(info) return PallyPowerWarn.db.profile.taunt.sound end,
					set = function(info,v)
						PallyPowerWarn.db.profile.taunt.sound = v
						PallyPowerWarn:Play(v)
						end,
					values = {
						["ding"] = L["Ding"],
						["dong"] = L["Dong"],
						["dodo"] = L["Dodo"],
						["bell"] = L["Bell"],
						["none"] = L["None"],
					},
					disabled = function() return not PallyPowerWarn.db.profile.taunt.enable end,
					order = 704,
				},
				petheader = {
					order = 705,
					type = "header",
					name = L["Pet Taunts"],
				},
				pet = {
					name = L["Pet Taunts"],
					type = "toggle",
					desc = L["Track pet actual taunts"],
					get = function(info) return PallyPowerWarn.db.profile.taunt.pet end,
					set = function(info,v)
						PallyPowerWarn.db.profile.taunt.pet = v
					end,
					disabled = function() return not PallyPowerWarn.db.profile.taunt.enable end,
					order = 706,
				},
				petaggro = {
					name = L["Pet Aggro"],
					type = "toggle",
					desc = L["Track pet aggro abilities"],
					get = function(info) return PallyPowerWarn.db.profile.taunt.petaggro end,
					set = function(info,v)
						PallyPowerWarn.db.profile.taunt.petaggro = v
					end,
					disabled = function() return not PallyPowerWarn.db.profile.taunt.enable end,
					order = 707,
				},
				noisypet = {
					name = L["Noisy Pet"],
					type = "toggle",
					desc = L["Notify each time instead of once per combat"],
					get = function(info) return PallyPowerWarn.db.profile.taunt.noisypet end,
					set = function(info,v)
						PallyPowerWarn.db.profile.taunt.noisypet = v
					end,
					disabled = function() return not PallyPowerWarn.db.profile.taunt.enable end,
					order = 708,
				},
				display = {
					name = L["Display"],
					type = 'group',
					desc = L["Settings for how to display the message."], 
					args = {
						color = {
							name = L["Color"],
							type = 'color',
							desc = L["Sets the color of the text when displaying messages."],
							get = function(info)
							local v = PallyPowerWarn.db.profile.taunt.display.color
								return v.r,v.g,v.b
								end,
							set = function(info,r,g,b) PallyPowerWarn.db.profile.taunt.display.color = {r=r,g=g,b=b} end
						},
						scroll = {
							type = 'toggle',
							name = L["Scroll output"],
							desc = L["Toggle showing messages in scrolling text. (Settings are in the 'Output Category')"],
							get = function(info) return PallyPowerWarn.db.profile.taunt.display.scroll end,
							set = function(info,t) PallyPowerWarn.db.profile.taunt.display.scroll = t end,
						},
						frames = {
							type = 'toggle',
							name = L["Frames output"],
							desc = L["Toggle whether the message will show up in the frames. (Settings are in the 'General Display' category.)"],
							get = function(info) return PallyPowerWarn.db.profile.taunt.display.frames end,
							set = function(info,t) PallyPowerWarn.db.profile.taunt.display.frames = t end,
						},
						time = {
							--hidden = true,
							name = L["Time to display message"],
							type = "range",
							desc = L["Set the time the message will be displayed (5=default)"],
							get = function(info) return PallyPowerWarn.db.profile.taunt.display.time end,
							set = function(info,v)
								PallyPowerWarn.db.profile.taunt.display.time = v
							end,
							step = 1,
							min = 1,
							max = 20,
						},
					}
				},
			}
		},
		beacon = {
			name = L["Beacon"],
			type = 'group',
			desc = L["Settings for Beacon of Light."],
			order = 800,
			args = {
				enable = {
					name = L["Enable"],
					type = "toggle",
					desc = L["Toggle Beacon tracker"],
					get = function(info) return PallyPowerWarn.db.profile.beacon.enable end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.enable = v
						PallyPowerWarn:LoadEvents()
					end,
					order = 801,
				},
				lock = {
					name = L["Lock tracker"],
					type = "toggle",
					desc = L["Lock Beacon tracker."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.lock end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.lock = v
						PallyPowerWarn:beacon_SetButtonMode()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 802,
				},
				tooltip = {
					name = L["Disable tooltip"],
					type = "toggle",
					desc = L["Disable Beacon tracker tooltip."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.notooltip end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.notooltip = v
					end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 803,
				},
				button = {
					name = L["Button only"],
					type = "toggle",
					desc = L["Show only the Beacon button."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.button end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.button = v
						PallyPowerWarn:beacon_SetButtonMode()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 804,
				},
				bgcolor = {
					name = L["Color"],
					type = "toggle",
					desc = L["Change the color of the background."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.bgcolor end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.bgcolor = v
						PallyPowerWarn:beacon_BuffCheck()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 805,
				},
				flash = {
					name = L["Flash"],
					type = "toggle",
					desc = L["Flash frame when expired."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.flash end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.flash = v
						PallyPowerWarn:beacon_BuffCheck()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 806,
				},
				resetpos = {
					name = L["Reset Position"],
					type = "execute",
					desc = L["Reset Position"],
					func = "ResetBeaconPos",
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 807,
				},
				whenheader = {
					order = 808,
					type = "header",
					name = L["When to check"],
				},
				readycheck = {
					name = L["Ready Check"],
					type = "toggle",
					desc = L["Notify on ready check."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.readycheck end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.readycheck = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 809, 
				},
				entercombat = {
					name = L["Enter Combat"],
					type = "toggle",
					desc = L["Notify when entering combat."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.entercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.entercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 810,
				},
				aftercombat = {
					name = L["After Combat"],
					type = "toggle",
					desc = L["Notify after the end of combat."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.aftercombat end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.aftercombat = v
						PallyPowerWarn:LoadEvents()
					end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 811,
				},
				disableheader = {
					order = 812,
					type = "header",
					name = L["Disable checks"],
				},
				mounted = {
					name = L["No Mounted"],
					type = "toggle",
					desc = L["Disable notifications when mounted."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.mounted end,
					set = function(info,v) PallyPowerWarn.db.profile.beacon.mounted = v end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 813,
				},
				vehicle = {
					name = L["No Vehicle"],
					type = "toggle",
					desc = L["Disable notifications when in a vehicle."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.vehicle end,
					set = function(info,v) PallyPowerWarn.db.profile.beacon.vehicle = v end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 814,
				},
				otherheader = {
					order = 815,
					type = "header",
					name = L["Other Options"],
				},
				alert = {
					name = L["Alert when fading"],
					type = "toggle",
					desc = L["Alert me when Beacon fades from my target."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.alert end,
					set = function(info,v)
						PallyPowerWarn.db.profile.beacon.alert = v
					end,
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 816,
				},
				sound = {
					name = L["Sound"],
					type = "select",
					desc = L["Play a sound when Beacon fades from my target."],
					get = function(info) return PallyPowerWarn.db.profile.beacon.sound end,
					set = function(info,v) PallyPowerWarn.db.profile.beacon.sound = v	end,
					values = {
						["ding"] = L["Ding"],
						["dong"] = L["Dong"],
						["dodo"] = L["Dodo"],
						["bell"] = L["Bell"],
						["none"] = L["None"],
					},
					disabled = function() return not PallyPowerWarn.db.profile.beacon.enable end,
					order = 817,
				},
				display = {
					name = L["Display"],
					type = 'group',
					desc = L["Settings for how to display the message."], 
					args = {
						color = {
							name = L["Color"],
							type = 'color',
							desc = L["Sets the color of the text when displaying messages."],
							get = function(info)
							local v = PallyPowerWarn.db.profile.beacon.display.color
								return v.r,v.g,v.b
								end,
							set = function(info,r,g,b) PallyPowerWarn.db.profile.beacon.display.color = {r=r,g=g,b=b} end
						},
						scroll = {
							type = 'toggle',
							name = L["Scroll output"],
							desc = L["Toggle showing messages in scrolling text. (Settings are in the 'Output Category')"],
							get = function(info) return PallyPowerWarn.db.profile.beacon.display.scroll end,
							set = function(info,t) PallyPowerWarn.db.profile.beacon.display.scroll = t end,
						},
						frames = {
							type = 'toggle',
							name = L["Frames output"],
							desc = L["Toggle whether the message will show up in the frames. (Settings are in the 'General Display' category.)"],
							get = function(info) return PallyPowerWarn.db.profile.beacon.display.frames end,
							set = function(info,t) PallyPowerWarn.db.profile.beacon.display.frames = t end,
						},
						time = {
							--hidden = true,
							name = L["Time to display message"],
							type = "range",
							desc = L["Set the time the message will be displayed (5=default)"],
							get = function(info) return PallyPowerWarn.db.profile.beacon.display.time end,
							set = function(info,v)
								PallyPowerWarn.db.profile.beacon.display.time = v
							end,
							step = 1,
							min = 1,
							max = 20,
							order = 111,
						},
					}
				},
			}
		},
	}
}

