--[[
$Id: PallyPowerWarn.lua 50 2012-03-13 01:15:07Z stassart $

Author: Trelis @ Proudmoore
(pally (a) stassart o org)

Copyright 2010-2012 Benjamin Stassart

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

]]

-- Disable PallyPowerWarn if the player is not a paladin
if select(2, UnitClass('player')) ~= "PALADIN" then
	DisableAddOn("PallyPowerWarn")
	return
end

local L = LibStub("AceLocale-3.0"):GetLocale("PallyPowerWarn")
local AceAddon = LibStub("AceAddon-3.0")

-- Functions marked with "*" are used by PallyPowerWarn
-- AceConsole: Print*, Printf, RegisterChatCommand, UnregisterChatCommand,
--		GetArgs
-- AceEvent: RegisterEvent*, UnregisterEvent, UnregisterAllEvents*
-- AceTimer: ScheduleTimer*, ScheduleRepeatingTimer, CancelTimer, CancelAllTimers, TimeLeft
-- LibSink: Pour*, RegisterSink*, SetSinkStorage, GetSinkAce2OptionsDataTable,
--		GetSinkAce3OptionsDataTable
PallyPowerWarn = AceAddon:NewAddon("PallyPowerWarn",
				"AceConsole-3.0",
				"AceEvent-3.0",
				"AceTimer-3.0",
				"LibSink-2.0")

local meta_version = GetAddOnMetadata("PallyPowerWarn","Version")
-- This doesn't get updated if this file isn't updated
-- local revision = ("$Revision: 50 $"):sub(12, -3)
local revision = GetAddOnMetadata("PallyPowerWarn","X-Curse-Packaged-Version")
PallyPowerWarn.DisplayVersion = ("v%s-%s"):format(meta_version, revision)

-- Add configmode support (i.e. OneButtonConfig)
-- Create the global table if it does not exist yet
CONFIGMODE_CALLBACKS = CONFIGMODE_CALLBACKS or {}
-- Declare our handler
CONFIGMODE_CALLBACKS["PallyPowerWarn"] = function(action)
	if action == "ON" then
		PallyPowerWarn.configMode = true
		PallyPowerWarn:UpdateLock()
	elseif action == "OFF" then
		PallyPowerWarn.configMode = false
		PallyPowerWarn:UpdateLock()
	end
end

-- Default Options for PallyPowerWarn
PallyPowerWarn.defaults = {
   profile = {
	display = {
		chat = true,
		number = 0,
		error = false,
		frame = true,
		fontSize = 18,
		fontFace = L["FRIZQT__.TTF"],
		fontEffect = L["OUTLINE"],
		lock = true,
	},
	blessing = {
		enable = true,
		readycheck = true,
		entercombat = false,
		aftercombat = true,
		mounted = true,
		vehicle = true,
		combat = false,
		pvp = false,
		bg = false,
		arena = true,
		wg = false,
		instance = true,
		raid = true,
		world = true,
		wrong = true,
		frequency = 5,
		sound = "bell",
		display = {
			color = { r=1, g=1, b=1 },
			scroll = false,
			frames = true,
			time = 5,
		},
	},
	seal = {
		enable = true,
		readycheck = true,
		entercombat = true,
		aftercombat = true,
		mounted = true,
		vehicle = true,
		combat = false,
		wrong = false,
		frequency = 5,
		sound = "dong",
		display = {
			color = { r=0.7, g=0.7, b=1 },
			scroll = false,
			frames = true,
			time = 5,
		},
	},
	rf = {
		enable = false,
		tankspec = "prot",
		readycheck = true,
		entercombat = true,
		aftercombat = true,
		mounted = true,
		vehicle = true,
		frequency = 5,
		sound = "dong",
		display = {
			color = { r=0.7, g=0.5, b=0 },
			scroll = false,
			frames = true,
			time = 5,
		},
	},
	aura = {
		enable = true,
		readycheck = true,
		entercombat = true,
		aftercombat = true,
		mounted = true,
		vehicle = true,
		bg = false,
		arena = false,
		wg = false,
		instance = true,
		raid = true,
		world = false,
		wrong = "crusader",
		overwritten = true,
		frequency = 5,
		sound = "dong",
		display = {
			color = { r=0.7, g=1, b=0.7 },
			scroll = false,
			frames = true,
			time = 5,
		},
	},
	announce = {
		interrupt = false,
		intchan = "none",
		dispel = false,
		dispelpm = false,
		dispelchan = "none",
		hof = false,
		hofpm = false,
		hofchan = "none",
		hoftext = "SPELL cast on TARGET",
		hop = false,
		hoppm = false,
		hopchan = "none",
		hoptext = "SPELL cast on TARGET",
		sac = false,
		sacpm = false,
		sacchan = "none",
		sactext = "SPELL cast on TARGET",
		salv = false,
		salvpm = false,
		salvchan = "none",
		salvtext = "SPELL cast on TARGET",
		aveng = false,
		avengchan = "none",
		avengtext = "SPELL gives you wings!",
		plea = false,
		pleachan = "none",
		pleatext = "SPELL activated",
		zeal = false,
		zealchan = "none",
		zealtext = "SPELL, the cure for sanity!",
		shield = false,
		shieldchan = "none",
		shieldtext = "SPELL activated. Hearthing!",
		am = false,
		amchan = "none",
		amtext = "SPELL activated AURA.",
		loh = false,
		lohpm = false,
		lohchan = "none",
		lohtext = "SPELL healed TARGET for AMOUNT!",
		dg = false,
		dgchan = "none",
		dgtext = "SPELL. Now with lower raid damage!",
		dp = false,
		dpchan = "none",
		dptext = "SPELL! Now with extra awe for your pleasure!",
		ardent = false,
		adchan = "none",
		adtext = "SPELL 1-up activated!",
		adproc = false,
		adprocchan = "none",
		adproctext = "SPELL healed for AMOUNT!",
		goak = false,
		goakchan = "none",
		goaktext = "SPELL activated",
		tankspec = "prot",
		target = true,
		frequency = 5,
		sound = "none",
		loc = {
		 	bgannounce = "none",
		 	arenaannounce = true,
		 	partyannounce = true,
		 	raidannounce = true,
			worldannounce = true,
		},
		display = {
			color = { r=1, g=0.5, b=0.7 },
			scroll = false,
			frames = true,
			time = 5,
		},
	},
	taunt = {
		enable = false,
		own = false,
		other = false,
		sound = "none",
		pet = false,
		petaggro = false,
		noisypet = false,
		display = {
			color = { r=1, g=0.7, b=0.7 },
			scroll = false,
			frames = true,
			time = 5,
		},
	},
	beacon = {
		enable = false,
		lock = false,
		notooltip = false,
		button = false,
		bgcolor = true,
		flash = false,
		readycheck = true,
		entercombat = true,
		aftercombat = false,
		mounted = true,
		vehicle = true,
		alert = true,
		sound = "none",
		display = {
			color = { r=1, g=1, b=0.5 },
			scroll = false,
			frames = true,
			time = 5,
		},
	},
	soundchannel = "Master",
   }
}

-- Sounds for alerts
local sounds = {
	ding = "Sound\\Doodad\\BellTollAlliance.wav",
	dong = "Sound\\Doodad\\BellTollHorde.wav",
	bell = "AuctionWindowClose",
	dodo = "igQuestFailed",
}

-- COMBAT_LOG_EVENT events that are tracked
local trackedEvents = {}

-- All Blessing Spell IDs
local blessing_ids = {
	[20217] = true, -- "Blessing of Kings",
	[79063] = true, -- "Blessing of Kings",
	[19740] = true, -- "Blessing of Might",
	[79102] = true, -- "Blessing of Might",
}

-- All Seal Spell IDs
local seal_ids = {
	[20154] = true, -- "Seal of Righteousness",
	[20165] = true, -- "Seal of Insight",
	[31801] = true, -- "Seal of Truth",
	[20164] = true, -- "Seal of Justice",
}

-- All Aura Spell IDs
local aura_ids = {
	[465] = true, -- "Devotion Aura",
	[7294] = true, -- "Retribution Aura",
	[19746] = true, -- "Concentration Aura",
	[19891] = true, -- "Resistance Aura",
	[32223] = true, -- "Crusader Aura",
}

-- Taunt Spells
local taunt_ids = {
	-- Warior
	[355] = true, -- Taunt
	[1161] = true, -- Challenging Shout

	-- Death Knight
	[49576] = true, -- Death Grip
	[56222] = true, -- Dark Command

	-- Paladin
	[62124] = true, -- Hand of Reckoning
	[31789] = true, -- Righteous Defense

	-- Druid
	[6795] = true, -- Growl
	[5209] = true, -- Challenging Roar

	-- Hunter
	[20736] = true, -- Distracting Shot

	-- Shaman
	[73684] = true, -- Unleash Earth

	-- Warlock
	[59671] = true, -- Challenging Howl
}

-- This is used for how the taunt is displayed
local aoetaunt_ids = {
	[1161] = true, -- Challenging Shout
	[5209] = true, -- Challenging Roar
	[59671] = true, -- Challenging Howl
	-- Pet
	[63900] = true, -- Thunderstomp (damage+threat, hunter pet talent)
	[58857] = true, -- Twin Howl (True taunt, Spirit Wolves)
}

local pettaunt_ids = {
	[53477] = true, -- Taunt (True taunt, Hunter pet talent)
	[17735] = true, -- Suffering (True taunt, Voidwalker)
	[58857] = true, -- Twin Howl (True taunt, Spirit Wolves)
}

local petaggro_ids = {
	[2649] = true, -- Growl (Increases threat, generic hunter pet)
	[63900] = true, -- Thunderstomp (damage+threat, hunter pet talent)
}

-- Interrupt IDs
local interrupt_ids = {
	-- Avenger's Shield
	-- Avenger's Shield only generates a SPELL_INTERRUPT event sometimes
	-- but I am not sure of a good solution to when the event is missing
	[31935] = true,
	-- Rebuke
	[96231] = true,
	-- Blood Elf Arcane Torrent
	[32747] = true,
	-- [28730] = true,
}

-- Spell ids for Spell Names
-- This is used to get localized names for spells
local name_ids = {
	["Cleanse"] = 4987,
	["Hand of Freedom"] = 1044,
	["Hand of Protection"] = 1022,
	["Hand of Sacrifice"] = 6940,
	["Hand of Salvation"] = 1038,
	["Avenging Wrath"] = 31884,
	["Divine Plea"] = 54428,
	["Zealotry"] = 85696,
	["Divine Shield"] = 642,
	["Aura Mastery"] = 31821,
	["Lay on Hands"] = 633,
	["Divine Guardian"] = 70940,
	["Divine Protection"] = 498,
	["Ardent Defender"] = 31850,
	["Beacon of Light"] = 53563,
	-- Prot version
	["Guardian of Ancient Kings"] = 86659,
}

-- Hand spell ids
local hand_ids = {
	-- Hand of Freedom
	[1044] = true,
	-- Hand of Protection
	[1022] = true,
	-- Hand of Sacrifice
	[6940] = true,
	-- Hand of Salvation
	[1038] = true,
}

-- Totem IDs
-- Totems that have identical effect as auras
local Totems = {
	[0] = "",
	[1] = GetSpellInfo(8072), -- Stoneskin Totem overwrites Devotion Aura
	[2] = "", -- Retribution Aura
	[3] = "", -- Concentration Aura - Totem of Tranquil Mind does not overwrite it
	[4] = "", -- Resistance Aura
	[5] = "", -- Crusader Aura
	[6] = "",
}

-- Localized names
local lN = {}

-- Track the last alert of a given type
local AlertTimes = {}

-- Players that have had buffs fade
local BuffFades = {}

-- Crusader Aura
local CrusaderSpell = GetSpellInfo(32223)

-- Righteous Fury
local RFSpell = GetSpellInfo(25780)

-- Pet taunts per combat
local CombatPetTaunt = {}

-- Beacon
local beacon_id
local beacon_target = "-"
local beacon_oldtarget
local beacon_casttarget
local beacon_lockdown = false

-- OnInitialize is called when the addon is loaded
function PallyPowerWarn:OnInitialize()
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	-- Attach to configuration settings
	self.db = LibStub("AceDB-3.0"):New("PallyPowerWarnDB",
			PallyPowerWarn.defaults, "profile")
	-- Load options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PallyPowerWarn",
		PallyPowerWarn.options, {"ppwarn", "ppw"} )

	-- Add options to Blizzard's Addon Interface
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("PallyPowerWarn",
		"PallyPowerWarn")

	-- Configure what the default options are
	self.db:RegisterDefaults(self.defaults)

	-- Display version number when loading

	self:Print(self.DisplayVersion.." Loaded. Use /ppw for options.")

	-- Announcements enabled?
	-- Can be disabled by zone
	self.announce = true

	-- Current zone
	-- Used to check for PvP zones
	self.zonetype = "none"

	-- ConfigMode (OneButtonConfig) off by default
	self.configMode = false

	-- Loaded?
	self.loaded = false

	self:InitializeBeacon()
	self:GetLocaleSpells()

	-- Register Scrolling Combat Text
	self:RegisterSink("PPW", "PallyPowerWarn", nil, "SinkPrint")
end

-- Called when the user enables the addon
function PallyPowerWarn:OnEnable()
	self:LoadEvents()
	-- self:PrintAssignments()
end

-- Called when the user disables the addon
function PallyPowerWarn:OnDisable()
	self:UnregisterAllEvents()
end

function PallyPowerWarn:GetLocaleSpells()
	lN["Avenging Wrath"] = GetSpellInfo(name_ids["Avenging Wrath"])
	lN["Divine Plea"] = GetSpellInfo(name_ids["Divine Plea"])
	lN["Zealotry"] = GetSpellInfo(name_ids["Zealotry"])
	lN["Divine Shield"] = GetSpellInfo(name_ids["Divine Shield"])
	lN["Divine Guardian"] = GetSpellInfo(name_ids["Divine Guardian"])
	lN["Divine Protection"] = GetSpellInfo(name_ids["Divine Protection"])
	lN["Ardent Defender"] = GetSpellInfo(name_ids["Ardent Defender"])
	lN["Beacon of Light"] = GetSpellInfo(name_ids["Beacon of Light"])
	lN["Guardian of Ancient Kings"] = GetSpellInfo(name_ids["Guardian of Ancient Kings"])
end

function PallyPowerWarn:InitializeBeacon()
	self.beacon = {}
	self.beacon = {
		enabled = false,
		TimerEvent = nil,
		elapsed = 0,
		flashing = false,
	}
end

-- Events handled by PallyPowerWarn
function PallyPowerWarn:LoadEvents()
	-- Disable all previous events from this addon
	self:UnregisterAllEvents()
	
	trackedEvents = {}

	-- Initially register this event
	-- Can't check spec until this event
	-- if (not self.loaded) then
	-- 	self:RegisterEvent("PLAYER_ALIVE")
	-- end

	-- Always monitor the combatlog
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

	-- Detect fading buffs
	trackedEvents["SPELL_AURA_REMOVED"] = true

	-- Dispel
	if self.db.profile.announce.dispel then
		trackedEvents["SPELL_DISPEL"] = true
	end

	-- Interrupt
	if self.db.profile.announce.interrupt then
		trackedEvents["SPELL_INTERRUPT"] = true
	end

	-- Hand of Freedom
	-- Hand of Protection
	-- Hand of Sacrifice
	-- Hand of Salvation
	if self.db.profile.announce.hof or
	   self.db.profile.announce.hop or
	   self.db.profile.announce.sac or
	   self.db.profile.announce.salv then
		trackedEvents["SPELL_AURA_APPLIED"] = true
	end

	-- Spell cast announcements
	if self.db.profile.announce.dg or
	   self.db.profile.announce.dp or
	   self.db.profile.announce.ardent or
	   self.db.profile.announce.goak or
	   self.db.profile.announce.aveng or
	   self.db.profile.announce.plea or
	   self.db.profile.announce.zeal or
	   self.db.profile.announce.shield or
	   self.db.profile.announce.am then
		self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	end

	-- Lay on Hands
	if self.db.profile.announce.loh or
	   self.db.profile.announce.adproc then
		trackedEvents["SPELL_HEAL"] = true
	end

	-- Can disable announcements by zone
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:ZoneCheck();

	-- Taunt Tracker
	if self.db.profile.taunt.enable then
		trackedEvents["SPELL_AURA_APPLIED"] = true
		trackedEvents["SPELL_CAST_SUCCESS"] = true
	end

	-- Ready Check
	if (self.db.profile.blessing.enable and self.db.profile.blessing.readycheck) or
	   (self.db.profile.seal.enable and self.db.profile.seal.readycheck) or
	   (self.db.profile.rf.enable and self.db.profile.rf.readycheck) or
	   (self.db.profile.aura.enable and self.db.profile.aura.readycheck) or
	   (self.db.profile.beacon.enable and self.db.profile.beacon.readycheck) then
		self:RegisterEvent("READY_CHECK")
	end
	
	-- Enter combat
	if (self.db.profile.blessing.enable and self.db.profile.blessing.entercombat) or
	   (self.db.profile.seal.enable and self.db.profile.seal.entercombat) or
	   (self.db.profile.rf.enable and self.db.profile.rf.entercombat) or
	   (self.db.profile.aura.enable and self.db.profile.aura.entercombat) or
	   (self.db.profile.beacon.enable and self.db.profile.beacon.entercombat) then
		self:RegisterEvent("PLAYER_REGEN_DISABLED")
	end

	-- After combat
	if (self.db.profile.blessing.enable and self.db.profile.blessing.aftercombat) or
	   (self.db.profile.seal.enable and self.db.profile.seal.aftercombat) or
	   (self.db.profile.rf.enable and self.db.profile.rf.aftercombat) or
	   (self.db.profile.aura.enable and self.db.profile.aura.aftercombat) or
	   (self.db.profile.beacon.enable) or
	   (self.db.profile.taunt.enable and (self.db.profile.taunt.pet or
	    self.db.profile.taunt.petaggro)) then

		self:RegisterEvent("PLAYER_REGEN_ENABLED")
	end

	-- Custom Message Frame
	if (not self.msgFrame) and self.db.profile.display.frame then
		self:CreateCustomFrame()
	end

	-- Beacon tracker
	if self.db.profile.beacon.enable then
		self:RegisterEvent("LEARNED_SPELL_IN_TAB")
		self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
		self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
		self:RegisterEvent("UNIT_SPELLCAST_SENT")
		self:RegisterEvent("UNIT_AURA")

		self:FindBeaconId()
		if (not self.beaconFrame) then
			self:CreateBeaconFrame()
		end
		if self.beaconFrame then
		 	if (beacon_id) then
				self.beaconFrame:Show()
				self.beaconButton:Show()
		 	end
		end
	elseif self.beaconFrame then
		self.beaconFrame:Hide()
		self.beaconButton:Hide()
	end
end

function PallyPowerWarn:FindBeaconId()
	local i = 1
	beacon_id = nil
	while true do
		local spell, rank = GetSpellBookItemName(i, BOOKTYPE_SPELL)
		if (not spell) then
			break
		end
		if spell == lN["Beacon of Light"] then
			beacon_id = i
		end
		i = i + 1
	end
end

function PallyPowerWarn:LEARNED_SPELL_IN_TAB(event, Tab)
 	if self.loaded then
		if self.beaconFrame then
			self.beaconFrame:Hide()
		end
		if self.beaconButton then
			self.beaconButton:Hide()
		end
 		-- self:Print("detected a new spell")
		self:LoadEvents()
 	end
end

function PallyPowerWarn:ACTIVE_TALENT_GROUP_CHANGED(event, newGroup, prevGroup)
 	if self.loaded then
		if self.beaconFrame then
			self.beaconFrame:Hide()
		end
		if self.beaconButton then
			self.beaconButton:Hide()
		end
		-- self:Print("detected a talent spec change")
	 	self:LoadEvents()
	end
end

function PallyPowerWarn:SPELL_UPDATE_COOLDOWN()
	if beacon_id and self.beaconButton then
		local start, duration, enable = GetSpellCooldown(beacon_id, BOOKTYPE_SPELL );
		CooldownFrame_SetTimer(self.beaconButton.Cooldown, start, duration, enable );
	end
end

-- a1 - Unit casting the spell
-- a2 - Spell name
-- a3 - Spell rank
-- a4 = Spell target
function PallyPowerWarn:UNIT_SPELLCAST_SENT(event, a1, a2, a3, a4)
	if a1 ~= "player" then
		return
	end

	if self.db.profile.beacon.enable and a2 == lN["Beacon of Light"] then
		if a4 ~= nil then
			beacon_casttarget = a4
			-- Temporarily disable earth shield checking
			if self.beacon.enabled then
				self.beacon.enabled = false
				self:CancelTimer(self.beacon.TimerEvent, false)
			end
			self:ScheduleTimer(self.beacon_checkTarget, 1, self)
		end
	end
end

function PallyPowerWarn:beacon_checkTarget()
	if beacon_id and self.db.profile.beacon.enable and beacon_target ~= beacon_casttarget then
		-- Target changed
		local buff, _, _, count, _, _, expirationTime, source = UnitBuff(beacon_casttarget, lN["Beacon of Light"], nil, "player")

		if buff then
			if InCombatLockdown() then
				-- Target changed in combat
				self.beaconButton.texture:SetVertexColor(0.8,0,0,0.8)
				if self.beacon_lockdown == false then
					self.beacon_oldtarget = beacon_target
					self.beacon_lockdown = true
					-- self:Print("Beacon oldtarget " .. self.beacon_oldtarget)
				end
			end
			if self.beacon_lockdown and self.beacon_oldtarget == beacon_casttarget then
				-- Target changed back to original
				self.beaconButton.texture:SetVertexColor(1,1,1,1)
				self.beacon_lockdown = false
			end

			beacon_target = beacon_casttarget
			-- self:Print("Updating Beacon in beacon_checkTarget")
			-- Beacon can only be cast in raid
			-- if UnitInParty(beacon_target) or UnitInRaid(beacon_target) then
			self:beacon_SetTarget(beacon_target)
			self:beacon_SetText("-")
			self:beacon_BuffCheck()
			self.beacon.enabled = true
			self.beacon.TimerEvent = self:ScheduleRepeatingTimer(self.beacon_BuffCheck, 10, self)
		end
	elseif beacon_id and self.db.profile.beacon.enable then
		-- Target has not changed
		local buff, _, _, count, _, _, expirationTime, source = UnitBuff(beacon_casttarget, lN["Beacon of Light"], nil, "player")

		if buff then
			-- Beacon never changed targets, re-enable beacon checking
			if not self.beacon.enabled then
				self.beacon.enabled = true
				self.beacon.TimerEvent = self:ScheduleRepeatingTimer(self.beacon_BuffCheck, 10, self)
				self:beacon_BuffCheck()
			end
		end
	end
end

-- a1 = Unit
function PallyPowerWarn:UNIT_AURA(event, a1)
	if beacon_id and self.db.profile.beacon.enable and (a1 == beacon_target or a1 == "player") then
		-- self:Print("Updating Beacon in UNIT_AURA")
		self:beacon_BuffCheck()

		-- Multiple beacons can be on the same target, so need to filter buffs with
		-- player as the source
		local buff, _, _, count, _, _, expirationTime, source = UnitBuff(beacon_target, lN["Beacon of Light"], nil, "player")
		-- self:Print("Beacon source: " .. source)

		if self.beacon.enabled and not buff then
			self.beacon.enabled = false
			self:CancelTimer(self.beacon.TimerEvent, false)
			local b_target_name = UnitName(beacon_target)
			if not b_target_name then
				b_target_name = L["Unknown"]
			end
			if self.db.profile.beacon.alert then
				self:Message(L["Beacon faded from "] .. b_target_name, "beacon")
			end
		end
	end
end

function PallyPowerWarn:beacon_BuffCheck()
	if (not beacon_target) or beacon_target == "-" then
		self:beacon_SetText("-")
		-- Don't warn when there is no target
		self:beacon_UpdateBG(true)
	else
		-- Multiple beacons can be on the same target, so need to filter buffs with
		-- player as the source
		local buff, _, _, count, _, _, expirationTime = UnitBuff(beacon_target, lN["Beacon of Light"], nil, "player")

		if buff == lN["Beacon of Light"] then
			if expirationTime == nil then
				expirationTime = "|cff999999Unavailable|r"
			else
				expirationTime = math.ceil((expirationTime - GetTime()) / 60) .. L[" min"]
			end

			self:beacon_SetText(expirationTime)
			self:beacon_UpdateBG(true)
		else
			self:beacon_SetText("-")
			self:beacon_UpdateBG(false)
		end
	end
end

-- Update Background color/flash for Earth Shield frame
function PallyPowerWarn:beacon_UpdateBG(flag)
	if (self.db.profile.beacon.button) then
		if (self.db.profile.beacon.lock) then
			self.beaconFrame:SetBackdropColor(0,0,0,0)
		else
			self:beacon_UpdateBGColor(flag)
		end
	else
		self:beacon_UpdateBGColor(flag)
	end
	self:beacon_UpdateBGAlpha(flag)
end

function PallyPowerWarn:IsEShieldCheckValid()
	local num_party = GetNumPartyMembers()
	local num_raid = GetNumRaidMembers()

	if (not UnitExists(beacon_target)) then
		return false
	elseif (not UnitInRange(beacon_target)) then
		return false
	elseif num_party < 1 and num_raid < 2 then
		-- Don't flash when solo
		return false
	else
		return true
	end
end

function PallyPowerWarn:beacon_UpdateBGColor(flag)
	if (self.db.profile.beacon.bgcolor and self:IsEShieldCheckValid()) then
		if (flag) then
			-- black
			self.beaconFrame:SetBackdropColor(0.1,0.1,0.1,0.9)
		else
			-- red
			self.beaconFrame:SetBackdropColor(1,0.1,0.1,0.9)
		end
	else
		self.beaconFrame:SetBackdropColor(0.1,0.1,0.1,0.9)
	end
end

-- Flash iteration time, fading in and fading out
local FLASH_TIME = 0.7
-- Time to fade in or out
local HALF_FLASH_TIME = FLASH_TIME / 2
local INVERSE_HALF_FLASH_TIME = 1 / HALF_FLASH_TIME

-- UIFrameFlash causes taint, so have to create own function
function PallyPowerWarn:beacon_UpdateBGAlpha(flag)
	-- if (self.db.profile.beacon.button) then
	if (self.db.profile.beacon.flash and not flag and self:IsEShieldCheckValid()) then
		if (not self.beacon.flashing) then
			self.beacon.flashing = true
			self.beacon.elapsed = 0
			self.beaconFrame:SetScript("OnUpdate",
				function(self, elapsed)
					local e = PallyPowerWarn.beacon.elapsed + elapsed
					PallyPowerWarn.beacon.elapsed = e
					local alpha = (e % FLASH_TIME) * INVERSE_HALF_FLASH_TIME
					if alpha > 1 then
						alpha = 2 - alpha
					end
 					PallyPowerWarn.beaconFrame:SetAlpha(alpha)
				end
			)
		end
	else
	 	self.beaconFrame:SetAlpha(1)
		-- Cancel any running flash script
		if (self.beacon.flashing) then
			self.beaconFrame:SetScript("OnUpdate", nil)
			self.beacon.flashing = false
		end
	end
end

-- Can't check spec prior to this event
-- function PallyPowerWarn:PLAYER_ALIVE(event)
	-- self:Print("login complete")
--	self:LoadEvents()
-- end

-- If set to raid, and in a party instead reverts to party
-- If set to raid or party and not in a party, does not output
function PallyPowerWarn:ChatChannel(chat)
	if chat == "RAID" and not UnitInRaid("player") then
		chat = "PARTY"
	end
	if chat == "PARTY" and GetNumPartyMembers() == 0 then
		chat = "none"
	end

	-- Currently in beta /say and /yell do not work in instances
	-- if self.zonetype == "party" and
	--    (chat == "SAY" or chat == "YELL") then
	-- 	if GetNumPartyMembers() > 0 then
	-- 		chat = "PARTY"
	-- 	end
	-- end

	-- self:Print("ChatChannel: " .. chat)
	return chat;
end

-- Process combat log events
function PallyPowerWarn:COMBAT_LOG_EVENT_UNFILTERED(e,
   timestamp,
   event,
   hideCaster,
   sourceGUID,
   sourceName,
   sourceFlags,
   sourceRaidFlags,
   destGUID,
   destName,
   destFlags,
   destRaidFlags,
   ...)
	local substring

	-- If paladin is dead, don't issue warnings
	if (UnitIsDeadOrGhost("player")) then
		return
	end

	-- End if it isn't an event we care about
	if not trackedEvents[event] then
		-- self:Print("NOT tracked")
		-- self:Print(timestamp, event, sourceGUID, sourceName,
		--		sourceFlags, destGUID, destName, destFlags)
		-- self:Print(...)
		return
	end

	if event == "SPELL_AURA_REMOVED" then
		-- self:Print(sourceGUID .. " " .. UnitGUID("player"))
		-- self:Print("sid: " .. select(1,...))
		if self.db.profile.blessing.enable and
		   sourceGUID == UnitGUID("player") and blessing_ids[select(1,...)] then
			if self:IsBlessingCheckEnabled() then
				-- Was it replaced by a new blessing?
				-- When a player dies, the buffs are removed right before
				-- they die, so we need to delay checking to see if they died
				BuffFades[destName] = select(2,...)
				self:ScheduleTimer(self.CheckFades, 1, self)
			end
		end
		if self.db.profile.seal.enable and
		   sourceGUID == UnitGUID("player") and seal_ids[select(1,...)] then
			if not (self.db.profile.seal.combat and InCombatLockdown()) then
				-- When a player dies, the buffs are removed right before
				-- they die, so we need to delay checking to see if they died
				self:ScheduleTimer(self.CheckSeal, 1, self)
			end
		end
		if self.db.profile.aura.enable and
		   sourceGUID == UnitGUID("player") and aura_ids[select(1,...)] then
			-- When a player dies, the buffs are removed right before
			-- they die, so we need to delay checking to see if they died
			self:ScheduleTimer(self.CheckAura, 1, self)
		end
	elseif event == "SPELL_AURA_APPLIED" then
		if hand_ids[select(1,...)] then
			self:HandEvent(sourceGUID, sourceName, destGUID, destName, select(1,...), select(2,...))
		elseif self.db.profile.taunt.enable and self:CheckTaunt(sourceGUID, select(1,...)) then
			-- Unleash Earth can only be checked for with SPELL_AURA_APPLIED
			-- The Unleash Elements cast is the same no matter which weapon enchant
			if select(1,...) == 73684 then
				local msg = string.format(L["%s taunted %s using %s"], sourceName, destName, select(2,...))
				self:Message(msg, "taunt")
			end
		end
	elseif (event == "SPELL_CAST_SUCCESS") then
		if self.db.profile.taunt.enable and self:CheckTaunt(sourceGUID, select(1,...)) then
			local msg = ""
			-- Righteous Defense is special
			if select(1,...) == 31789 then
				msg = string.format(L["%s AoE taunted off of %s using %s"], sourceName, destName, select(2,...))
			elseif aoetaunt_ids[select(1,...)] then
				msg = string.format(L["%s AoE taunted using %s"], sourceName, select(2,...))
			else
				msg = string.format(L["%s taunted %s using %s"], sourceName, destName, select(2,...))
			end
			self:Message(msg, "taunt")
		end
	elseif event == "SPELL_INTERRUPT" then
		if self.db.profile.announce.interrupt and sourceGUID == UnitGUID("player") and interrupt_ids[select(1,...)] then
			-- Do I need to use a limited Message/Announce to prevent spamming for arcane torrent interrupting multiple mobs?
			self:LimitedMessage(string.format(L["Interrupted: %s"],select(5,...)),"announce",self.db.profile.announce.frequency)
			if self:ChatChannel(self.db.profile.announce.intchan) ~= "none" then
				-- Can't use GetSpellLink since it isn't in spellbook
				local spellstring = "\124cff71d5ff\124Hspell:" .. select(4,...) .. "\124h[" .. select(5,...) .. "]\124h\124r"

				if self.db.profile.announce.target then
					self:LimitedAnnounce(string.format(L["Interrupted: %s"] .. " (%s)",spellstring,destName),self.db.profile.announce.frequency,select(2,...),self:ChatChannel(self.db.profile.announce.intchan))
				else
					self:LimitedAnnounce(string.format(L["Interrupted: %s"],spellstring),self.db.profile.announce.frequency,select(2,...),self:ChatChannel(self.db.profile.announce.intchan))
				end
			end
		end
	elseif event == "SPELL_DISPEL" then
		if self.db.profile.announce.dispel and sourceGUID == UnitGUID("player") and select(1,...) == name_ids["Cleanse"] then
			self:Message(string.format(L["Dispel: "] .. "%s (%s)",select(5,...),destName),"announce")
			-- Can't use GetSpellLink since it isn't in spellbook
			local spellstring = "\124cff71d5ff\124Hspell:" .. select(4,...) .. "\124h[" .. select(5,...) .. "]\124h\124r"
			if self:ChatChannel(self.db.profile.announce.dispelchan) ~= "none" then
				if self.db.profile.announce.target then
					self:Announce(string.format(L["Dispel: "] .. "%s (%s)",spellstring,destName),self:ChatChannel(self.db.profile.announce.dispelchan))
				else
					self:Announce(L["Dispel: "] .. spellstring,self:ChatChannel(self.db.profile.announce.dispelchan))
				end
			end

			if self.db.profile.announce.dispelpm then
				self:Whisper(L["Dispel: "] .. spellstring,destName)
			end
		end
	elseif event == "SPELL_HEAL" then
		if self.db.profile.announce.loh and sourceGUID == UnitGUID("player") and select(1,...) == name_ids["Lay on Hands"] then
			-- Replace SPELL in string with the actual spell
			local spellstring = GetSpellLink(name_ids["Lay on Hands"])
			local substring = string.gsub(self.db.profile.announce.lohtext, "SPELL", spellstring)
			-- Replace AMOUNT in string with the amount healed
			-- effective = amount - overheal
			local eff = select(4,...) - select(5,...)
			if (eff < 0) then
				eff = 0
			end
			substring = string.gsub(substring, "AMOUNT", tostring(eff))

			if self:ChatChannel(self.db.profile.announce.lohchan) ~= "none" then
				-- Replace TARGET in string with target
				local chatstring = string.gsub(substring, "TARGET", destName)
				self:Announce(chatstring,self:ChatChannel(self.db.profile.announce.lohchan))
			end

			if self.db.profile.announce.lohpm then
				-- Replace TARGET with "you" for whispers
				local pmstring = string.gsub(substring, "TARGET", L["you"])
				self:Whisper(pmstring,destName)
			end
		elseif self.db.profile.announce.adproc and sourceGUID == UnitGUID("player") and select(1,...) == 66235 then
			if self:ChatChannel(self.db.profile.announce.adprocchan) ~= "none" then
				-- Replace SPELL in string with the actual spell
				local spellstring = GetSpellLink(66235)
				local substring = string.gsub(self.db.profile.announce.adproctext, "SPELL", spellstring)
				-- Replace AMOUNT in string with the amount healed
				-- effective = amount - overheal
				local eff = select(4,...) - select(5,...)
				if (eff < 0) then
					eff = 0
				end
				substring = string.gsub(substring, "AMOUNT", tostring(eff))

				self:Announce(substring,self:ChatChannel(self.db.profile.announce.adprocchan))
			end
		end
	end
	
	-- self:Print("tracked")
	-- self:Print(timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags)
	-- self:Print(...)
end

-- Hand of Freedom
-- Hand of Protection
-- Hand of Sacrifice
-- Hand of Salvation
function PallyPowerWarn:HandEvent(sourceGUID, sourceName, destGUID, destName, spellid, spellname)
	if sourceGUID == UnitGUID("player") then
		if self.db.profile.announce.hof and spellid == name_ids["Hand of Freedom"] then
			if self.announce then
				-- Replace TARGET in string with the actual target
				local substring = string.gsub(self.db.profile.announce.hoftext, "TARGET", destName)
				self:SpellCastAnnounce(spellid, spellname, substring, self.db.profile.announce.hofchan)
			end
			if self.db.profile.announce.hofpm then
				-- Replace SPELL in string with the actual spell
				local spellstring = GetSpellLink(spellid)
				local substring = string.gsub(self.db.profile.announce.hoftext, "SPELL", spellstring)
				-- Replace TARGET with "you" for whispers
				substring = string.gsub(substring, "TARGET", L["you"])
				self:Whisper(substring,destName)
			end
		elseif self.db.profile.announce.hop and spellid == name_ids["Hand of Protection"] then
			if self.announce then
				-- Replace TARGET in string with the actual target
				local substring = string.gsub(self.db.profile.announce.hoptext, "TARGET", destName)
				self:SpellCastAnnounce(spellid, spellname, substring, self.db.profile.announce.hopchan)
			end
			if self.db.profile.announce.hoppm then
				-- Replace SPELL in string with the actual spell
				local spellstring = GetSpellLink(spellid)
				local substring = string.gsub(self.db.profile.announce.hoptext, "SPELL", spellstring)
				-- Replace TARGET with "you" for whispers
				substring = string.gsub(substring, "TARGET", L["you"])
				self:Whisper(substring,destName)
			end
		elseif self.db.profile.announce.sac and spellid == name_ids["Hand of Sacrifice"] then
			if self.announce then
				-- Replace TARGET in string with the actual target
				local substring = string.gsub(self.db.profile.announce.sactext, "TARGET", destName)
				self:SpellCastAnnounce(spellid, spellname, substring, self.db.profile.announce.sacchan)
			end
			if self.db.profile.announce.sacpm then
				-- Replace SPELL in string with the actual spell
				local spellstring = GetSpellLink(spellid)
				local substring = string.gsub(self.db.profile.announce.sactext, "SPELL", spellstring)
				-- Replace TARGET with "you" for whispers
				substring = string.gsub(substring, "TARGET", L["you"])
				self:Whisper(substring,destName)
			end
		elseif self.db.profile.announce.salv and spellid == name_ids["Hand of Salvation"] then
			if self.announce then
				-- Replace TARGET in string with the actual target
				local substring = string.gsub(self.db.profile.announce.salvtext, "TARGET", destName)
				self:SpellCastAnnounce(spellid, spellname, substring, self.db.profile.announce.salvchan)
			end
			if self.db.profile.announce.salvpm then
				-- Replace SPELL in string with the actual spell
				local spellstring = GetSpellLink(spellid)
				local substring = string.gsub(self.db.profile.announce.salvtext, "SPELL", spellstring)
				-- Replace TARGET with "you" for whispers
				substring = string.gsub(substring, "TARGET", L["you"])
				self:Whisper(substring,destName)
			end
		end
	elseif destGUID == UnitGUID("player") then
		-- Someone else cast hand on the player
		self:Message(string.format(L["%s cast %s on you"],sourceName,spellname),"announce")
	end
end

-- a1 = Unit casting the spell
-- a2 = Spell name
-- a3 = Spell rank
function PallyPowerWarn:UNIT_SPELLCAST_SUCCEEDED(event,a1,a2,a3)
	if a1 ~= "player" then
		return
	end
	-- self:Print("UNIT_SPELLCAST_SUCCEEDED: a1: "..a1.." a2: "..a2)

	if self.db.profile.announce.dg and (a2 == lN["Divine Guardian"]) and self:CheckSpec(self.db.profile.announce.tankspec) then
		self:SpellCastAnnounce(name_ids["Divine Guardian"],a2,self.db.profile.announce.dgtext,self.db.profile.announce.dgchan)
	elseif self.db.profile.announce.dp and (a2 == lN["Divine Protection"]) and self:CheckSpec(self.db.profile.announce.tankspec) then
		self:SpellCastAnnounce(name_ids["Divine Protection"],a2,self.db.profile.announce.dptext,self.db.profile.announce.dpchan)
	elseif self.db.profile.announce.ardent and (a2 == lN["Ardent Defender"]) and self:CheckSpec(self.db.profile.announce.tankspec) then
		self:SpellCastAnnounce(name_ids["Ardent Defender"],a2,self.db.profile.announce.adtext,self.db.profile.announce.adchan)
	elseif self.db.profile.announce.goak and (a2 == lN["Guardian of Ancient Kings"]) and self:CheckSpec(self.db.profile.announce.tankspec) then
		self:SpellCastAnnounce(name_ids["Guardian of Ancient Kings"],a2,self.db.profile.announce.goaktext,self.db.profile.announce.goakchan)
	elseif self.db.profile.announce.aveng and (a2 == lN["Avenging Wrath"]) then
		self:SpellCastAnnounce(name_ids["Avenging Wrath"],a2,self.db.profile.announce.avengtext,self.db.profile.announce.avengchan)
	elseif self.db.profile.announce.plea and (a2 == lN["Divine Plea"]) then
		self:SpellCastAnnounce(name_ids["Divine Plea"],a2,self.db.profile.announce.pleatext,self.db.profile.announce.pleachan)
	elseif self.db.profile.announce.zeal and (a2 == lN["Zealotry"]) then
		self:SpellCastAnnounce(name_ids["Zealotry"],a2,self.db.profile.announce.zealtext,self.db.profile.announce.zealchan)
	elseif self.db.profile.announce.shield and (a2 == lN["Divine Shield"]) then
		self:SpellCastAnnounce(name_ids["Divine Shield"],a2,self.db.profile.announce.shieldtext,self.db.profile.announce.shieldchan)
	elseif self.db.profile.announce.am and (a2 == lN["Aura Mastery"]) then
		-- Handle Aura Mastery special
		-- have it display which aura
		if self:ChatChannel(self.db.profile.announce.amchan) ~= "none" then
			local aura = self:GetMyAura()

			-- Replace SPELL in string with the actual spell
			local spellstring = GetSpellLink(name_ids["Aura Mastery"])
			local aurastring = GetSpellLink(aura)

			local substring = string.gsub(self.db.profile.announce.amtext, "SPELL", spellstring)
			substring = string.gsub(substring, "AURA", aurastring)
			self:Announce(substring,self:ChatChannel(self.db.profile.announce.amchan))
		end
	end
end

function PallyPowerWarn:IsProt()
	local holy = select(5,GetTalentTabInfo(1))
	local prot = select(5,GetTalentTabInfo(2))
	local ret = select(5,GetTalentTabInfo(3))

	if (prot > holy and prot > ret) then
		return true
	else
		return false
	end
end

function PallyPowerWarn:IsRet()
	local holy = select(5,GetTalentTabInfo(1))
	local prot = select(5,GetTalentTabInfo(2))
	local ret = select(5,GetTalentTabInfo(3))

	if (ret > holy and ret > prot) then
		return true
	else
		return false
	end
end

-- Conditions to check for Blessing Fade
function PallyPowerWarn:IsBlessingCheckEnabled()
	-- Check when mounted?
	if (self.db.profile.blessing.mounted and IsMounted()) then
		return false
	end

	-- Check when in vehicle?
	if (self.db.profile.blessing.vehicle and UnitInVehicle("player")) then
		return false
	end

	-- Check when in combat?
	if (self.db.profile.blessing.combat and InCombatLockdown()) then
		return false
	end

	-- Check when PvP flagged?
	if (self.db.profile.blessing.pvp) then
		if UnitIsPVP("player") or UnitIsPVPFreeForAll("player") then
			return false
		end
	end

	-- Do we check in this location?
	if (not self:IsBlessingCheckLocation()) then
		return false
	end

	return true
end

-- Check all players that have had buffs fade
function PallyPowerWarn:CheckFades()
	for k, v in pairs (BuffFades) do
		-- self:Print("k = "..k..", v = "..v)
		self:CheckBuffFade(k, v)
	end
	BuffFades = {}
end

-- Check if a buff faded
function PallyPowerWarn:CheckBuffFade(unitid, oldBlessing)
	local name = UnitName(unitid)
	local hasbuff, haskings, hasmight, visible, isdead = self:HasBuffs(unitid)
	if hasbuff == false and visible == true and isdead == false then
		local blessing = self:GetMyBlessing(unitid)
		if not blessing then
			self:LimitedMessage(
			  oldBlessing .. L[" faded from "] ..name,
			  "blessing",
			  self.db.profile.blessing.frequency)
		end
	end
end

-- Called to open the configuration
function PallyPowerWarn:OpenConfig()
	if InterfaceOptionsFrame_OpenToCategory then
		InterfaceOptionsFrame_OpenToCategory("PallyPowerWarn");
	else
		InterfaceOptionsFrame_OpenToFrame("PallyPowerWarn");
	end
end

-- Show the version number
function PallyPowerWarn:ShowVersion()
	self:Print(self.DisplayVersion)
end

-- Perform Checks
function PallyPowerWarn:PerformChecks()
	self:CheckBlessings()
	self:CheckSeal()
	self:CheckRF()
	self:CheckAura()
end

-- Ready Check event
function PallyPowerWarn:READY_CHECK()
	-- self:Print("READY_CHECK")
	if self.db.profile.blessing.readycheck then
		self:CheckBlessings()
	end
	if self.db.profile.seal.readycheck then
		self:CheckSeal()
	end
	if self.db.profile.rf.readycheck then
		self:CheckRF()
	end
	if self.db.profile.aura.readycheck then
		self:CheckAura()
	end
	if self.db.profile.beacon.readycheck then
		self:CheckBeacon()
	end
end

-- Enter combat event
function PallyPowerWarn:PLAYER_REGEN_DISABLED()
	-- self:Print("PLAYER_REGEN_DISABLED")
	if self.db.profile.blessing.entercombat then
		self:CheckBlessings()
	end
	if self.db.profile.seal.entercombat then
		self:CheckSeal()
	end
	if self.db.profile.rf.entercombat then
		self:CheckRF()
	end
	if self.db.profile.aura.entercombat then
		self:CheckAura()
	end
	if self.db.profile.beacon.entercombat then
		self:CheckBeacon()
	end
end

-- Leave combat event
function PallyPowerWarn:PLAYER_REGEN_ENABLED()
	-- self:Print("PLAYER_REGEN_ENABLED")
	if self.db.profile.blessing.aftercombat then
		self:CheckBlessings()
	end
	if self.db.profile.seal.aftercombat then
		self:CheckSeal()
	end
	if self.db.profile.rf.aftercombat then
		self:CheckRF()
	end
	if self.db.profile.aura.aftercombat then
		self:CheckAura()
	end
	if self.db.profile.beacon.aftercombat then
		self:CheckBeacon()
	end

	if (beacon_id and self.db.profile.beacon.enable) then
		-- Cannot update who the button casts on while in combat
		-- Update as soon as combat is over
		-- self:Print("Updating Beacon in PLAYER_REGEN_ENABLED")
	 	self:beacon_SetTarget(beacon_target)
		self:beacon_BuffCheck()
	end
end

-- Has the paladin fully buffed blessings?
function PallyPowerWarn:CheckBlessings()
	local missing_kings = 0
	local missing_might = 0
	local missing_range = 0
	local wrong_buffs = 0
	local num_raid = GetNumRaidMembers()
	local max_raid = 0
	local unit_prefix
	local israid = 0
	local PlayersMissingKings = {}
	local PlayersMissingMight = {}
	local PlayersMissingRange = {}
	local PlayersWrongBuffs = {}

	-- 1 = Kings
	-- 2 = Might
	local current_buff = 0

	local kingsName = GetSpellInfo(20217)
	local kingsString = GetSpellLink(20217)
	local mightName = GetSpellInfo(19740)
	local mightString = GetSpellLink(19740)

	-- Enabled?
	if (not self.db.profile.blessing.enable) then
		return
	end

	-- Don't warn if dead
	if (UnitIsDeadOrGhost("player")) then
		return
	end

	-- Check when mounted?
	if (self.db.profile.blessing.mounted and IsMounted()) then
		return
	end

	-- Check when in vehicle?
	if (self.db.profile.blessing.vehicle and UnitInVehicle("player")) then
		return
	end

	-- Check when PvP flagged?
	if (self.db.profile.blessing.pvp) then
		if UnitIsPVP("player") or UnitIsPVPFreeForAll("player") then
			return
		end
	end

	-- Do we check in this location?
	if (not self:IsBlessingCheckLocation()) then
		return
	end

	if num_raid < 1 then
		-- party
		max_raid = MAX_PARTY_MEMBERS
		unit_prefix = "party"
		israid = 0
	else
		-- raid
		max_raid = MAX_RAID_MEMBERS
		unit_prefix = "raid"
		israid = 1
	end
	-- self:Print("max_raid: " .. max_raid)

	local hasbuff
	local haskings
	local hasmight
	local visible
	local isdead
	local blessing
	local mykings
	local mymight

	-- player
	hasbuff, haskings, hasmight, visible, isdead = self:HasBuffs("player")
	if visible and not isdead then
		blessing, mykings, mymight = self:GetMyBlessing("player")
		if (mykings) then
			current_buff = 1
		elseif (mymight) then
			current_buff = 2
		end

		local name = UnitName("player")
		if (not haskings) then
			PlayersMissingKings[name] = true
			missing_kings = missing_kings + 1
		end
		if (not hasmight) then
			PlayersMissingMight[name] = true
			missing_might = missing_might + 1
		end
		if (not hasbuff and blessing) then
			-- Has a blessing from the player,
			-- but not the PallyPower assigned one
			wrong_buffs = wrong_buffs + 1
			PlayersWrongBuffs[name] = true
			-- self:Print(name.." has wrong buff")
		end
	end

	for i = 1, max_raid do
		local unitid
		-- Check Player
		unitid = unit_prefix..i
		-- self:Print(unitid)
		if unitid and UnitExists(unitid) then
			hasbuff, haskings, hasmight, visible, isdead = self:HasBuffs(unitid)

			-- Server lag will sometimes cause newly spawned units
			-- to show up as "Unknown"
			-- Can't check buffs properly on "Unknown" units
			local name = UnitName(unitid)

			if visible and not isdead and name ~= L["Unknown"] then
				blessing, mykings, mymight = self:GetMyBlessing("player")
				if (mykings) then
					current_buff = 1
				elseif (mymight) then
					current_buff = 2
				end

				if (not haskings) then
					PlayersMissingKings[name] = true
					missing_kings = missing_kings + 1
				end
				if (not hasmight) then
					PlayersMissingMight[name] = true
					missing_might = missing_might + 1
				end
				if (not hasbuff and blessing) then
					-- Has a blessing from the player,
					-- but not the PallyPower assigned one
					wrong_buffs = wrong_buffs + 1
					PlayersWrongBuffs[name] = true
					-- self:Print(name.." has wrong buff")
				end
			end

			if not visible and not hasbuff and not isdead and name ~= L["Unknown"] then
				missing_range = missing_range + 1
				PlayersMissingRange[name] = true
			end
			
		end
	end

	if missing_might > 0 then
		-- If currently providing Kings, don't warn about missing Might
		if (current_buff ~= 1) then
			local message = L["Missing "] .. mightString .. ": " .. missing_might
			self:LimitedMessage(message, "blessing",
				self.db.profile.blessing.frequency)
		end

		-- Print players missing Might
		local player_string = ""
		for k, v in pairs (PlayersMissingMight) do
			if player_string == "" then
				player_string = k
			else
				player_string = player_string..", "..k
			end
		end
		self:ChatMessage(L["Players missing Might: "]..player_string,
			"blessing")
	end

	if missing_kings > 0 then
		-- If currently providing Might, don't warn about missing Kings
		if (current_buff ~= 2) then
			local message = L["Missing "] .. kingsString .. ": " .. missing_kings
			self:LimitedMessage(message, "blessing",
				self.db.profile.blessing.frequency)
		end

		-- Print players missing Kings
		local player_string = ""
		for k, v in pairs (PlayersMissingKings) do
			if player_string == "" then
				player_string = k
			else
				player_string = player_string..", "..k
			end
		end
		self:ChatMessage(L["Players missing Kings/MOTW: "]..player_string,
			"blessing")
	end

	if missing_range > 0 then
		-- Out of range players missing buffs
		local player_string = ""
		for k, v in pairs (PlayersMissingRange) do
			if player_string == "" then
				player_string = k
			else
				player_string = player_string..", "..k
			end
		end
		self:ChatMessage(L["Players out of range: "]..player_string,
			"blessing")
	end

	-- Print what buff the player is currently providing
	-- self:Print("current_buff: "..tostring(current_buff))
	if (not current_buff or current_buff == 0) then
		-- self:Print("In current_buff 0")
		local message = L["Neither providing Kings nor Might"]
		self:ChatMessage(message, "blessing")
	elseif (current_buff == 1) then
		local message = L["Currently providing "] .. kingsString
		self:ChatMessage(message, "blessing")
	elseif (current_buff == 2) then
		local message = L["Currently providing "] .. mightString
		self:ChatMessage(message, "blessing")
	end

	if wrong_buffs > 0 then
		if self.db.profile.blessing.wrong then
			local message = L["Wrong Buffs"] .. ": " .. wrong_buffs
			self:LimitedMessage(message, "blessing",
				self.db.profile.blessing.frequency)
		end

		-- Print players missing buffs
		local player_string = ""
		for k, v in pairs (PlayersWrongBuffs) do
			if player_string == "" then
				player_string = k
			else
				player_string = player_string..", "..k
			end
		end
		self:ChatMessage(L["Players with wrong buffs: "]..player_string,
			"blessing")
	end
end

-- Does the pally have a seal?
function PallyPowerWarn:CheckSeal()
	-- Enabled?
	if (not self.db.profile.seal.enable) then
		return
	end

	-- Don't warn if dead
	if (UnitIsDeadOrGhost("player")) then
		return
	end

	-- Check when mounted?
	if (self.db.profile.seal.mounted and IsMounted()) then
		return
	end

	-- Check when in vehicle?
	if (self.db.profile.seal.vehicle and UnitInVehicle("player")) then
		return
	end

	-- Check if PallyPower is running
	local pp = false
	if (_G.PallyPower) then
		pp = true
	end

	local seal_spell = ""
	if (pp) then
		seal_spell = PallyPower.Seals[PallyPower.db.profile.seal]
	end

	local spell, expire = self:GetSeal()
	-- self:Print ("seal_spell: "..tostring(seal_spell)..", spell: "..tostring(spell))

	if (not spell) then
		self:LimitedMessage(L["Missing Seal"], "seal",
			self.db.profile.seal.frequency)
	elseif pp and (seal_spell ~= spell) and self.db.profile.seal.wrong then
		self:LimitedMessage(L["Wrong Seal"], "seal",
			self.db.profile.seal.frequency)
	elseif not pp and self.db.profile.seal.wrong then
		-- Check if it is the right seal by spec
		if (self:IsProt()) then
			local truth = GetSpellInfo(31801)
			if (truth ~= spell) then
				self:LimitedMessage(L["Wrong Seal"], "seal",
					self.db.profile.seal.frequency)
			end
		elseif (self:IsRet()) then
			local truth = GetSpellInfo(31801)
			local right = GetSpellInfo(20154)
			if (truth ~= spell and right ~= spell) then
				self:LimitedMessage(L["Wrong Seal"], "seal",
					self.db.profile.seal.frequency)
			end
		else
			-- Holy
			local insight = GetSpellInfo(20165)
			if (insight ~= spell) then
				self:LimitedMessage(L["Wrong Seal"], "seal",
					self.db.profile.seal.frequency)
			end
		end
	end
end

-- Does the pally have Righteous Fury?
function PallyPowerWarn:CheckRF()
	-- Enabled?
	if (not self.db.profile.rf.enable) then
		return
	end

	-- Don't warn if dead
	if (UnitIsDeadOrGhost("player")) then
		return
	end

	-- Check when mounted?
	if (self.db.profile.rf.mounted and IsMounted()) then
		return
	end

	-- Check when in vehicle?
	if (self.db.profile.rf.vehicle and UnitInVehicle("player")) then
		return
	end

	local rf_on = UnitBuff("player", RFSpell)

	-- Check if PallyPower is running
	if (not _G.PallyPower) then
		-- Do we check RF in this spec?
		if (self:CheckSpec(self.db.profile.rf.tankspec) and not rf_on) then
			self:LimitedMessage(L["Missing Righteous Fury"], "rf",
				self.db.profile.rf.frequency)
		-- elseif (not isprot and rf_on) then
		-- 	self:LimitedMessage(L["Righteous Fury is on"], "rf",
		-- 		self.db.profile.rf.frequency)
		end

		return
	end

	-- Is Righteous Fury enabled in PallyPower?
	local rf_enabled = PallyPower.db.profile.rf

	-- Is Righteous Fury on?
	if rf_enabled and not rf_on then
		self:LimitedMessage(L["Missing Righteous Fury"], "rf",
			self.db.profile.rf.frequency)
	elseif not rf_enabled and rf_on then
		self:LimitedMessage(L["Righteous Fury is on"], "rf",
			self.db.profile.rf.frequency)
	end
end

-- This doesn't work on login until the PLAYER_ALIVE event fires
-- until the character is loaded, the spec detects as 0/0/0
function PallyPowerWarn:CheckSpec(spec)

	local active_spec = GetActiveTalentGroup()
	-- Number of talent points in each spec
	local holy = select(5,GetTalentTabInfo(1))
	local prot = select(5,GetTalentTabInfo(2))
	local ret = select(5,GetTalentTabInfo(3))

	-- self:Print("CheckSpec active: "..active_spec.." balance: "..balance.." feral: "..feral.." resto: "..resto)

	if (spec == "both") then
		return true
	elseif (spec == "primary" and active_spec == 1) then
		return true
	elseif (spec == "secondary" and active_spec == 2) then
		return true
	elseif (spec == "prot" and prot > holy and prot > ret) then
		-- Prot spec has the most talent points
		return true
	else
		return false
	end
end

-- Does the pally have an aura?
function PallyPowerWarn:CheckAura()
	-- Enabled?
	if (not self.db.profile.aura.enable) then
		return
	end

	-- Don't warn if dead
	if (UnitIsDeadOrGhost("player")) then
		return
	end

	-- Check when mounted?
	if (self.db.profile.aura.mounted and IsMounted()) then
		return
	end

	-- Check when in vehicle?
	if (self.db.profile.aura.vehicle and UnitInVehicle("player")) then
		return
	end

	-- Check if PallyPower is running
	local pp = false
	if (_G.PallyPower) then
		pp = true
	end

	-- If we are checking for wrong aura, then get the assigned aura
	local auraSpell = ""
	local aura_on = false
	local totem_on = false
	if (pp) then
		local auraID = PallyPower.db.profile.aura
		auraSpell = PallyPower.Auras[auraID]

		-- Is the assigned aura on?
		-- It could be overwritten by another paladin with a more powerful version
		aura_on = UnitBuff("player", auraSpell)

		-- Is it overwritten by a totem?
		local totemSpell = Totems[auraID]
		totem_on = UnitBuff("player", totemSpell)
	else
		if (self:GetMyAura()) then
			aura_on = true
		end
	end

	if (not totem_on) then
		-- Even if PallyPower assigned aura isn't overwritten
		-- Check if Devotion Aura is selected and if Stoneskin totem is up
		-- Form 1 is Devotion
		local devotion_name = GetSpellInfo(465)
		local texture, name, devotion_on = GetShapeshiftFormInfo(1)
		if (name and devotion_name ~= name) then
			self:Print("PallyPowerWarn error: "..tostring(devotion_name).."!="..tostring(name))
		elseif (devotion_on) then
			local totemSpell = Totems[1]
			totem_on = UnitBuff("player", totemSpell)
		end
	end

	-- Do I have any aura on?
	local spell = self:GetMyAura()

	-- self:Print ("auraSpell: "..tostring(auraSpell)..", spell: "..tostring(spell))
	if (not aura_on and not totem_on and not spell) then
		self:LimitedMessage(L["Missing Aura"], "aura",
			self.db.profile.aura.frequency)
	elseif (self:IsAuraCheckLocation()) then
		if (auraSpell ~= spell) and (spell == CrusaderSpell) and
				(self.db.profile.aura.wrong == "crusader") then
			self:LimitedMessage(L["Wrong Aura"], "aura",
				self.db.profile.aura.frequency)
		elseif pp and (auraSpell ~= spell) and
				(self.db.profile.aura.wrong == "pp") then
			self:LimitedMessage(L["Wrong Aura"], "aura",
				self.db.profile.aura.frequency)
		elseif (not spell) and
				(aura_on or totem_on) and
				(self.db.profile.aura.overwritten) then
			self:LimitedMessage(L["Aura Overwritten"], "aura",
				self.db.profile.aura.frequency)
		end
	end
end

function PallyPowerWarn:CheckBeacon()
	-- Enabled?
	if (not self.db.profile.beacon.enable) then
		return
	end

	-- Don't warn if dead
	if (UnitIsDeadOrGhost("player")) then
		return
	end

	-- Check when mounted?
	if (self.db.profile.beacon.mounted and IsMounted()) then
		return
	end

	-- Check when in vehicle?
	if (self.db.profile.beacon.vehicle and UnitInVehicle("player")) then
		return
	end

	-- Solo? Don't alert about missing Beacon if solo.
	local num_party = GetNumPartyMembers()
	local num_raid = GetNumRaidMembers()
	if num_party < 1 and num_raid < 2 then
		return
	end

	if beacon_id then
		local buff, _, _, count, _, _, expirationTime = UnitBuff(beacon_target, lN["Beacon of Light"], nil, "player")
		if not buff then
			self:Message(L["Missing Beacon of Light"], "beacon")
		end
	end
end

-- Has PallyPower assigned buff?
-- returns hasbuff, visible
function PallyPowerWarn:HasBuffs(unitid)
	local hasbuff = false
	local haskings = false
	local hasmight = false
	local visible = false

	if not unitid or not UnitExists(unitid) then
		return nil
	end

	-- Get Info on unit
	local class
	local name = UnitName(unitid)
	local isdead = UnitIsDeadOrGhost(unitid) == 1
	local kingsName = GetSpellInfo(20217)
	local mightName = GetSpellInfo(19740)
	local motwName = GetSpellInfo(1126)
	-- BM Hunter Pet version of Kings/MoTW
	local embraceName = GetSpellInfo(90363) -- Embrace of the Shale Spider

	-- Check if PallyPower is running
	local pp = false
	if (_G.PallyPower) then
		pp = true
	end

	 -- Check for Kings and might
	if (UnitBuff(unitid, kingsName) or
	    UnitBuff(unitid, motwName) or
	    UnitBuff(unitid, embraceName)) then
		haskings = true
	end
	if (UnitBuff(unitid, mightName)) then
		hasmight = true
	end

	local spellName

	-- Determine assigned buff
	if (pp) then
		spellName = PallyPower.Spells[PallyPower.db.profile.buff]

		if self:IsBuffActive(spellName, unitid) then
			hasbuff = true
		-- elseif (haskings and spellName == kingsName) then
		-- 	hasbuff = true
		elseif (haskings and hasmight) then
			-- Has both buffs paladins can provide
			hasbuff = true
		else
			hasbuff = false
		end
	else
		-- Am I providing a blessing?
		spellName = self:GetMyBlessing(unitid)

		if (spellName) then
			hasbuff = true
		else
			-- Not using PallyPower and not providing a blessing myself
			-- But are we fully Pally Buffed anyways?

			if (haskings and hasmight) then
				-- Has both buffs paladins can provide
				hasbuff = true
			else
				-- Missing a buff paladins can provide
				-- and this paladin is not providing a buff
				hasbuff = false
			end

			-- default Blessing of Might for visible
			spellName = GetSpellInfo(19740)
		end
	end

	-- self:Print(name.." assigned "..tostring(spellName))

	visible = IsSpellInRange(spellName, unitid) == 1

	-- self:Print(tostring(name)..": visible: "..tostring(visible)..", hasbuff: "..tostring(hasbuff)..", isdead: "..tostring(isdead))
	return hasbuff, haskings, hasmight, visible, isdead
end

-- IsPet
function PallyPowerWarn:IsPet(unitid)
	return not UnitIsPlayer(unitid) and
		(UnitPlayerControlled(unitid) or UnitPlayerOrPetInRaid(unitid))
end

-- Checks if unit has assigned buff
-- If it is cast by another paladin, that is fine.
-- Someone else buffed for me.
function PallyPowerWarn:IsBuffActive(spellName, unitID)
	if UnitBuff(unitID, spellName) then
		local buffName, _, _, _, _, buffDuration,
			buffExpire = UnitBuff(unitID, spellName)
		return buffExpire, buffDuration, buffName
	else
		return nil
	end
end

-- Get the paladin's current blessing on target
-- returns nil if no blessing
function PallyPowerWarn:GetMyBlessing(unitid)
	local kingsName = GetSpellInfo(20217)
	local mightName = GetSpellInfo(19740)
	local spell
	local mykings = false
	local mymight = false

	-- Go through all buffs on a unit until you find a blessing
	-- cast by the player (if any)
	local i = 1
	local buffName, _, _, _, _, _, _, _, _, _,
		buffID = UnitBuff(unitid, i, nil, "player")
	while buffName do
		if blessing_ids[buffID] and buffName then
			spell = buffName
			break
		end
		i = i + 1
		buffName, _, _, _, _, _, _, _, _, _,
			buffID = UnitBuff(unitid, i, nil, "player")
	end
	if (spell == kingsName) then
		mykings = true
	end
	if (spell == mightName) then
		mymight = true
	end
	-- self:Print("GetMyBlessing, kings: "..tostring(mykings)..", might: "..tostring(mymight))
	return spell, mykings, mymight
end

-- Get the paladin's current seal
-- returns nil if no seal
function PallyPowerWarn:GetSeal()
	local spell
	local sealExpire = 9999
	local sealDuration = 30*60

	-- Go through all the player's self buffs until you find a seal (if any)
	local i = 1
	local buffName, _, _, _, _, buffDuration, buffExpire, _, _, _,
		buffID = UnitBuff("player", i, nil, "player")
	while buffExpire do
		if seal_ids[buffID] and buffExpire then
			spell = buffName
			sealExpire = buffExpire - GetTime()
			break
		end
		i = i + 1
		buffName, _, _, _, _, buffDuration, buffExpire, _, _, _,
			buffID = UnitBuff("player", i, nil, "player")
	end
	return spell, sealExpire, sealDuration
end

-- Get the paladin's current aura
-- returns nil if no aura
function PallyPowerWarn:GetMyAura()
	local index = GetShapeshiftForm()	
	if (index == 0) then
	 	return nil
	end

	local texture, name, active = GetShapeshiftFormInfo(index)
	-- self:Print("GetMyAura: "..name..", active: "..tostring(active)..", index: "..index)
	if (active) then
		-- This checks if the buff is provided by anyone
		if UnitBuff("player", name) then
			return name
		end
	end
	return nil
end

-- Check if this is a location where we check Blessings
function PallyPowerWarn:IsBlessingCheckLocation()
	local zone = GetRealZoneText()

	-- First check for Wintergrasp by zone text
	-- If warn in Wintergrasp is selected, it will warn
	-- even if World warnings are otherwise disabled
	if zone == L["ZONE_WG"] or
	   zone == L["ZONE_TOLB"] then
		if self.db.profile.blessing.wg then
			return true
		else
			return false
		end
	end

	local inInstance, instanceType = IsInInstance();

	-- Then check instance type
	if instanceType == "pvp" then
	 	if not self.db.profile.blessing.bg then
			return false
		end
	elseif instanceType == "arena" then
	 	if not self.db.profile.blessing.arena then
			return false
		end
	elseif instanceType == "party" then
		if not self.db.profile.blessing.instance then
			return false
		end
	elseif instanceType == "raid" then
 		if not self.db.profile.blessing.raid then
			return false
		end
	elseif not self.db.profile.blessing.world then
		-- "none"
		return false
	end

	return true
end

-- Check if this is a location where we check Auras
function PallyPowerWarn:IsAuraCheckLocation()
	local zone = GetRealZoneText()

	-- First check for Wintergrasp by zone text
	-- If warn in Wintergrasp is selected, it will warn
	-- even if World warnings are otherwise disabled
	if zone == L["ZONE_WG"] or
	   zone == L["ZONE_TOLB"] then
		if self.db.profile.aura.wg then
			return true
		else
			return false
		end
	end

	local inInstance, instanceType = IsInInstance();

	-- Then check instance type
	if instanceType == "pvp" then
	 	if not self.db.profile.aura.bg then
			return false
		end
	elseif instanceType == "arena" then
	 	if not self.db.profile.aura.arena then
			return false
		end
	elseif instanceType == "party" then
		if not self.db.profile.aura.instance then
			return false
		end
	elseif instanceType == "raid" then
	 	if not self.db.profile.aura.raid then
			return false
		end
	elseif not self.db.profile.aura.world then
		-- "none"
		return false
	end

	return true
end

-- Print PallyPower blessing assignments
-- This can be used to test that PallyPowerWarn is reading the PallyPower
-- assignments
-- /run PallyPowerWarn:PrintAssignments()
function PallyPowerWarn:PrintAssignments()

	-- Check if PallyPower is running
	if (not _G.PallyPower) then
		return
	end

	local playerName = UnitName("player")
	local spellName = PallyPower.Spells[PallyPower.db.profile.buff]
	self:Print(playerName ..": ".. tostring(spellName))
end

-- Don't display a message of this type more often than limit
function PallyPowerWarn:LimitedMessage(str, type, limit)
	local last_time = 0
	if AlertTimes[type] then
		last_time = AlertTimes[type]
	end

	if (limit == 0) or ((GetTime() - last_time) > limit) then
		AlertTimes[type] = GetTime()
		self:Message(str, type)
	end
end

function PallyPowerWarn:ChatMessage(str, type)
	local c
	if type == "blessing" then
		c = self.db.profile.blessing.display.color
	elseif type == "seal" then
		c = self.db.profile.seal.display.color
	elseif type == "rf" then
		c = self.db.profile.rf.display.color
	elseif type == "aura" then
		c = self.db.profile.aura.display.color
	elseif type == "taunt" then
		c = self.db.profile.aura.display.color
	elseif type == "beacon" then
		c = self.db.profile.aura.display.color
	end
	if (type == "blessing" and self.db.profile.blessing.display.frames) or
	   (type == "seal" and self.db.profile.seal.display.frames) or
	   (type == "rf" and self.db.profile.rf.display.frames) or
	   (type == "aura" and self.db.profile.aura.display.frames) or
	   (type == "taunt" and self.db.profile.taunt.display.frames) or
	   (type == "beacon" and self.db.profile.beacon.display.frames) then
		-- Display Chat Message?
		if self.db.profile.display.chat then
			if self.db.profile.display.number == 0 then
				self:Print(str)
			else
				local chatframe = getglobal("ChatFrame" .. self.db.profile.display.number)
				chatframe:AddMessage(str, c.r, c.g, c.b)
			end
		end
	end
end

-- Display alerts to the user
function PallyPowerWarn:Message(str, type)
	local c, t
	local opt_sound
	if type == "blessing" then
		c = self.db.profile.blessing.display.color
		t = self.db.profile.blessing.display.time
		opt_sound = self.db.profile.blessing.sound
	elseif type == "seal" then
		c = self.db.profile.seal.display.color
		t = self.db.profile.seal.display.time
		opt_sound = self.db.profile.seal.sound
	elseif type == "rf" then
		c = self.db.profile.rf.display.color
		t = self.db.profile.rf.display.time
		opt_sound = self.db.profile.rf.sound
	elseif type == "aura" then
		c = self.db.profile.aura.display.color
		t = self.db.profile.aura.display.time
		opt_sound = self.db.profile.aura.sound
	elseif type == "taunt" then
		c = self.db.profile.taunt.display.color
		t = self.db.profile.taunt.display.time
		opt_sound = self.db.profile.taunt.sound
	elseif type == "beacon" then
		c = self.db.profile.beacon.display.color
		t = self.db.profile.beacon.display.time
		opt_sound = self.db.profile.beacon.sound
	end
	if (type == "blessing" and self.db.profile.blessing.display.frames) or
	   (type == "seal" and self.db.profile.seal.display.frames) or
	   (type == "rf" and self.db.profile.rf.display.frames) or
	   (type == "aura" and self.db.profile.aura.display.frames) or
	   (type == "taunt" and self.db.profile.taunt.display.frames) or
	   (type == "beacon" and self.db.profile.beacon.display.frames) then
		-- Display Chat Message?
		if self.db.profile.display.chat then
			if self.db.profile.display.number == 0 then
				self:Print(str)
			else
				local chatframe = getglobal("ChatFrame" .. self.db.profile.display.number)
				chatframe:AddMessage(str, c.r, c.g, c.b)
			end
		end
		-- Display to Blizzard Error frame?
		if self.db.profile.display.error then
			UIErrorsFrame:AddMessage(str, c.r, c.g, c.b, 1, t)
		end
		-- Display to Custom Frame?
		if self.db.profile.display.frame then
			self.msgFrame:SetTimeVisible(t)
			self.msgFrame:AddMessage(str, c.r, c.g, c.b, 1, t)
		end
	end
	if (type == "blessing" and self.db.profile.blessing.display.scroll) or
	   (type == "seal" and self.db.profile.seal.display.scroll) or
	   (type == "rf" and self.db.profile.rf.display.scroll) or
	   (type == "aura" and self.db.profile.aura.display.scroll) or
	   (type == "taunt" and self.db.profile.taunt.display.scroll) or
	   (type == "beacon" and self.db.profile.beacon.display.scroll) then
		-- Use LibSink to handle scrolling text
		self:Pour(str, c.r, c.g, c.b)
	end

	-- Play sound if one is enabled
	self:Play(opt_sound)
end

-- Play a sound
function PallyPowerWarn:Play(sound_arg)
	if (sound_arg == "none") then
		return
	end

	local sound = sounds[sound_arg]
	if (not sound) then
		return
	end

	if string.find(sound, "%\\") then
		PlaySoundFile(sound, PallyPowerWarn.db.profile.soundchannel)
	else
		PlaySound(sound, PallyPowerWarn.db.profile.soundchannel)
	end
end

-- sid = Spell Id
-- loc_spell = Localized Spell Name
-- Text = Text to display
-- Channel = Channel to announce to
function PallyPowerWarn:SpellCastAnnounce(sid, loc_spell, text, channel)
	if self:ChatChannel(channel) == "none" then
		return
	end

	-- Replace SPELL in string with the actual spell
	local spellstring = GetSpellLink(sid)
	local substring = string.gsub(text, "SPELL", spellstring)
	self:Announce(substring,self:ChatChannel(channel))
end

-- language and channel are optional
function PallyPowerWarn:LimitedAnnounce(msg, limit, spell, chatType, language, channel)
	local last_time = 0
	-- Group all announcements together
	if AlertTimes[spell] then
		last_time = AlertTimes[spell]
	end

	if limit and ((limit == 0) or ((GetTime() - last_time) > limit)) then
		AlertTimes[spell] = GetTime()
		self:Announce(msg, chatType, language, channel)
	end
end

-- language and channel are optional
-- If language is nil it is "ORCISH" for Horde and "COMMON" for Alliance
function PallyPowerWarn:Announce(msg, chatType, language, channel)
	-- self:Print("In Announce")
	if self.announce == false then
		return
	end

	-- Battlegrounds need to be handled specially
	if self.zonetype == "pvp" then
		if self.db.profile.announce.loc.bgannounce == "BATTLEGROUND" then
			chatType = "BATTLEGROUND"
		elseif self.db.profile.announce.loc.bgannounce == "PARTY" then
			if GetRealNumPartyMembers() > 0 then
				chatType = "PARTY"
			else
				return
			end
		elseif self.db.profile.announce.loc.bgannounce == "none" then
			return
		end
	end

	-- self:Print("announce: " .. tostring(self.announce) .. " chatType: " .. chatType)

	SendChatMessage(msg, chatType, language, channel);
end

-- Display alerts to the user
-- Language is optional:
-- If language is nil it is "ORCISH" for Horde and "COMMON" for Alliance
function PallyPowerWarn:Whisper(msg, target, language)
	SendChatMessage(msg, "WHISPER", language, target);
end

-- Scrolling Combat Text output
function PallyPowerWarn:SinkPrint(addon, message, r, g, b)
 	if not self.msgFrame then self:CreateCustomFrame() end
 	self.msgFrame:AddMessage(message, r, g, b, 1, UIERRORS_HOLD_TIME)
end

-- Create the output frame
function PallyPowerWarn:CreateCustomFrame()
	self.dragButton = CreateFrame("Button",nil,UIParent)
	self.dragButton.owner = self
	self.dragButton:Hide()
	self.dragButton:ClearAllPoints()
	self.dragButton:SetWidth(250)
	self.dragButton:SetHeight(20)
	
	if self.db.profile.display.x and self.db.profile.display.y then
		self.dragButton:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT",
			self.db.profile.display.x, self.db.profile.display.y)
	else 
		self.dragButton:SetPoint("TOP", UIErrorsFrame, "BOTTOM", 0, 0)
	end	
	
	self.dragButton:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	})
	self.dragButton:SetBackdropColor(0,0,0.3,.6)	
	
	self.dragButton:SetMovable(true)
	self.dragButton:RegisterForDrag("LeftButton")
	self.dragButton:SetScript("OnDragStart",
		function() PallyPowerWarn.dragButton:StartMoving() end )
	self.dragButton:SetScript("OnDragStop",
		function()
			PallyPowerWarn.dragButton:StopMovingOrSizing()
			self.db.profile.display.x = PallyPowerWarn.dragButton:GetLeft()
			self.db.profile.display.y = PallyPowerWarn.dragButton:GetTop()
		end
	)

	self.msgFrame = CreateFrame("MessageFrame")
	self.msgFrame.owner = self
	self.msgFrame:ClearAllPoints()
	self.msgFrame:SetWidth(400)
	self.msgFrame:SetHeight(75)
	self.msgFrame:SetPoint("TOP", self.dragButton, "TOP", 0, 0)
	self.msgFrame:SetInsertMode("TOP")
	self.msgFrame:SetFrameStrata("HIGH")
	self.msgFrame:SetToplevel(true)

	self.msgText = self.dragButton:CreateFontString(nil, "BACKGROUND",
		"GameFontNormalSmall")
	self.msgText:SetText("PallyPowerWarn Display")
	self.msgText:SetPoint("TOP", self.dragButton, "TOP", 0, -5)

	self:UpdateLock()
	self:UpdateFont()
	
	self.msgFrame:Show()
end

-- Update the font in the custom frame
function PallyPowerWarn:UpdateFont()
	if self.db.profile.display.frame and self.msgFrame then
		self.msgFrame:SetFont("Fonts\\" .. self.db.profile.display.fontFace,
			self.db.profile.display.fontSize,
			self.db.profile.display.fontEffect)
	end
end

-- Lock or unlock the custom frame
function PallyPowerWarn:UpdateLock()
	if self.db.profile.display.frame and self.msgFrame then
		if self.db.profile.display.lock and not self.configMode then
			self.dragButton:SetMovable(false)
			self.dragButton:RegisterForDrag()
			self.msgFrame:SetBackdrop(nil)
			self.msgFrame:SetBackdropColor(0,0,0,0)
			self.dragButton:Hide()
		else
			self.dragButton:Show()
			self.dragButton:SetMovable(true)
			self.dragButton:RegisterForDrag("LeftButton")
			self.msgFrame:SetBackdrop({
				bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			})
			self.msgFrame:SetBackdropColor(0,0,0.3,.3)
		end
	end
end

function PallyPowerWarn:CheckTaunt(sourceGUID, spellid)
	if (not self.db.profile.taunt.own and sourceGUID == UnitGUID("player")) or
	   (not self.db.profile.taunt.other and sourceGUID ~= UnitGUID("player")) then
		return false
	end

	-- Is it a pet taunt?
	if (pettaunt_ids[spellid] or petaggro_ids[spellid]) then
		-- Are we listing pet taunts?
		if (not self.db.profile.taunt.pet and pettaunt_ids[spellid]) or
		   (not self.db.profile.taunt.petaggro and petaggro_ids[spellid]) then
			return false
		end
		
		if (self.db.profile.taunt.noisypet) then
			-- No limit on how many times to output
			return true
		end

		-- Otherwise only say something the first time a pet does it per combat
		if CombatPetTaunt[sourceGUID] then
			return false
		else
			CombatPetTaunt[sourceGUID] = true
			return true
		end
	end

	if taunt_ids[spellid] then
		return true
	end
	return false
end

function PallyPowerWarn:PLAYER_ENTERING_WORLD()
	self:ZoneCheck();
	self.loaded = true
end

function PallyPowerWarn:ZoneCheck()
	local inInstance, instanceType = IsInInstance();

	self.announce = true
	self.zonetype = instanceType

	if instanceType == "pvp" then
	 	if self.db.profile.announce.loc.bgannounce == "none" then
			self.announce = false
		end
	elseif instanceType == "arena" then
		if self.db.profile.announce.loc.arenaannounce == false then
			self.announce = false
		end
	elseif instanceType == "party" then
	 	if self.db.profile.announce.loc.partyannounce == false then
			self.announce = false
		end
	elseif instanceType == "raid" then
	 	if self.db.profile.announce.loc.raidannounce == false then
			self.announce = false
		end
	else
	 	if self.db.profile.announce.loc.worldannounce == false then
			self.announce = false
		end
	end
end

function PallyPowerWarn:CreateBeaconFrame()
	if beacon_id == nil then
		return
	end
	
	self.beaconFrame = CreateFrame("Frame","PallyPowerWarn_beacon",UIParent)
	self.beaconFrame.owner = self
	self.beaconFrame:ClearAllPoints()
	self.beaconFrame:SetWidth(130)
	self.beaconFrame:SetHeight(36)
--	self.beaconFrame:SetFrameStrata("DIALOG")
	self.beaconFrame:SetFrameStrata("MEDIUM")
	self.beaconFrame:SetToplevel(true)
	self.beaconFrame:SetMovable(true)
	self.beaconFrame:RegisterForDrag("LeftButton")
	self.beaconFrame:SetScript("OnDragStart",
		function()
			if (not self.db.profile.beacon.lock) or self.configMode then
				PallyPowerWarn.beaconFrame:StartMoving()
			end
		end
	)
	self.beaconFrame:SetScript("OnDragStop",
		function()
			PallyPowerWarn.beaconFrame:StopMovingOrSizing()
			self.db.profile.beacon.x = PallyPowerWarn.beaconFrame:GetLeft()
			self.db.profile.beacon.y = PallyPowerWarn.beaconFrame:GetTop()
		end
	)
	
	if self.db.profile.beacon.x and self.db.profile.beacon.y then
		self.beaconFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.profile.beacon.x, self.db.profile.beacon.y)
	else 
		self.beaconFrame:SetPoint("BOTTOMRIGHT", UIParrent, "BOTTOMRIGHT", -200, 300)
	end	
	
	self.beaconHeader = self.beaconFrame:CreateFontString(nil, "OVERLAY")
	self.beaconHeader.owner = self
	
	self.beaconInfo = self.beaconFrame:CreateFontString(nil, "OVERLAY")
	self.beaconInfo.owner = self

	self:beacon_SetButtonMode()

	self.beaconButton = CreateFrame("Button", "PallyPowerWarn_beaconButton", self.beaconFrame, "SecureActionButtonTemplate")
	self.beaconButton.class = self
	self.beaconButton:SetPoint("TOPRIGHT", self.beaconFrame, "TOPLEFT", 0, 0)
	self.beaconButton:SetWidth(36)
	self.beaconButton:SetHeight(36)
--	self.beaconButton:SetFrameStrata("DIALOG")
	self.beaconButton:SetFrameStrata("MEDIUM")
	self.beaconButton:SetToplevel(true)
	self.beaconButton:HookScript("OnEnter", PallyPowerWarn.ShowTooltip)
	self.beaconButton:HookScript("OnLeave", PallyPowerWarn.HideTooltip)
	
	self.beaconButton.texture = self.beaconButton:CreateTexture(nil, "ARTWORK")
	self.beaconButton.texture:SetTexture(GetSpellTexture(lN["Beacon of Light"]))
	self.beaconButton.texture:SetPoint("TOPLEFT", self.beaconButton, "TOPLEFT", 3, -3)
	self.beaconButton.texture:SetPoint("BOTTOMRIGHT", self.beaconButton, "BOTTOMRIGHT", -3, 3)
	self.beaconButton.texture:SetTexCoord(0.10, 0.90, 0.10, 0.90)
	
	self.beaconButton_shadow = CreateFrame("Frame", "PallyPowerWarn_beaconButton_shadow", self.beaconButton)
	--frame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 1, edgeFile = "", edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0},})
	self.beaconButton_shadow:SetBackdrop({
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = true, tileSize = 8, edgeSize = 9,
			insets = { left = 2, right =2, top = 2, bottom = 2 }
		})
	self.beaconButton_shadow:SetBackdropColor(0.1, 0.1, 0.1, 0.9)
	self.beaconButton_shadow:SetAllPoints(self.beaconButton)
	self.beaconButton_shadow:SetFrameLevel(0)
	
	self.beaconButton.Cooldown = CreateFrame("Cooldown", "PallyPowerWarn_beaconButton_cooldown", self.beaconButton, "CooldownFrameTemplate");
	
	self.beacon_target = "target"
	self:beacon_SetTarget("target")
	
	self:beacon_SetText("-")

	self.beaconFrame:Show()
end

function PallyPowerWarn:ResetBeaconPos()
	local lock = false
	if self.db.profile.beacon.lock then
		lock = true
		self.db.profile.beacon.lock = false
	end
	self.db.profile.beacon.x = nil
	self.db.profile.beacon.y = nil
	self.beaconFrame:SetPoint("BOTTOMRIGHT", UIParrent, "BOTTOMRIGHT", -200, 300)
	if lock then
		self.db.profile.beacon.lock = true
	end
	self:beacon_SetButtonMode()
end

function PallyPowerWarn:beacon_SetButtonMode()
	self.beaconFrame:ClearAllPoints()
	if self.db.profile.beacon.x and self.db.profile.beacon.y then
		self.beaconFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.profile.beacon.x, self.db.profile.beacon.y)
	else 
		self.beaconFrame:SetPoint("BOTTOMRIGHT", UIParrent, "BOTTOMRIGHT", -200, 300)
	end

	if (PallyPowerWarn.db.profile.beacon.notooltip and
	    PallyPowerWarn.db.profile.beacon.lock and not PallyPowerWarn.configMode) then
		self.beaconFrame:EnableMouse(false)
	else
		self.beaconFrame:EnableMouse(true)
	end

	self.beaconFrame:Hide()
	if (self.db.profile.beacon.button) then
		self.beaconFrame:SetWidth(50)
		self.beaconFrame:SetHeight(36)
		if (PallyPowerWarn.db.profile.beacon.lock) then
			self.beaconFrame:SetBackdrop(nil)
			self.beaconFrame:SetBackdropColor(0,0,0,0)
		else
			self.beaconFrame:SetBackdrop({
					bgFile = "Interface/Tooltips/UI-Tooltip-Background",
					edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
					tile = true, tileSize = 8, edgeSize = 9,
					insets = { left = 2, right =2, top = 2, bottom = 2 }
				})
			self.beaconFrame:SetBackdropColor(0.1,0.1,0.1,0.9)
		end

		self.beaconInfo:SetFontObject(GameFontNormalLarge)
		self.beaconInfo:ClearAllPoints()
		self.beaconInfo:SetTextColor(1, 1, 1, 1)
		self.beaconInfo:SetWidth(50)
		self.beaconInfo:SetHeight(36)
		self.beaconInfo:SetPoint("TOP", self.beaconFrame, "TOP",0,0)
		self.beaconInfo:SetJustifyH("LEFT")
		self.beaconInfo:SetJustifyV("MIDDLE")

		self:beacon_SetText("-")
		self:beacon_BuffCheck()
		-- self:Print("SetButtonMode Hide")
		self.beaconFrame:Show()
		self.beaconHeader:Hide()
		self.beaconInfo:Show()
	else
		self.beaconFrame:SetWidth(130)
		self.beaconFrame:SetHeight(36)
		self.beaconFrame:SetBackdrop({
				bgFile = "Interface/Tooltips/UI-Tooltip-Background",
				edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
				tile = true, tileSize = 8, edgeSize = 9,
				insets = { left = 2, right =2, top = 2, bottom = 2 }
			})
		self.beaconFrame:SetBackdropColor(0.1,0.1,0.1,0.9)

		self.beaconHeader:SetFontObject(GameFontNormalSmall)
		self.beaconHeader:ClearAllPoints()
		self.beaconHeader:SetWidth(94)
		self.beaconHeader:SetHeight(12)
		self.beaconHeader:SetPoint("TOP", self.beaconFrame, "TOP",0,-2)
		self.beaconHeader:SetJustifyH("CENTER")
		self.beaconHeader:SetJustifyV("TOP")
		self.beaconHeader:SetText(L["Beacon Tracker"])

		self.beaconInfo:SetFontObject(GameFontNormalSmall)
		self.beaconInfo:ClearAllPoints()
		self.beaconInfo:SetTextColor(1, 1, 1, 1)
		self.beaconInfo:SetWidth(94)
		self.beaconInfo:SetHeight(24)
		self.beaconInfo:SetPoint("TOP", self.beaconHeader, "BOTTOM",0,4)
		self.beaconInfo:SetJustifyH("LEFT")
		self.beaconInfo:SetJustifyV("MIDDLE")

		self:beacon_SetText("-")
		self:beacon_BuffCheck()

		-- self:Print("SetButtonMode Show")
		self.beaconFrame:Show()
		self.beaconHeader:Show()
		self.beaconInfo:Show()
	end
end

-- Beacon of Light Tooltip
-- Do not use "self" in this function (lua error)
function PallyPowerWarn:ShowTooltip()
	if (PallyPowerWarn.db.profile.beacon.notooltip) then
		return
	end

	if (beacon_id and PallyPowerWarn.db.profile.beacon.enable) then
		GameTooltip:SetOwner(PallyPowerWarn.beaconButton,ANCHOR_LEFT)
		GameTooltip:SetSpellBookItem(beacon_id, BOOKTYPE_SPELL)
	end
end

function PallyPowerWarn:HideTooltip()
	GameTooltip:Hide()
end

function PallyPowerWarn:beacon_SetTarget(t)
	if not InCombatLockdown() then
		if (self.beaconButton) then
			self.beaconButton:SetAttribute("type1", "spell")
			self.beaconButton:SetAttribute("spell1", lN["Beacon of Light"])
			self.beaconButton:SetAttribute("unit1", t)
			self.beaconButton.texture:SetVertexColor(1,1,1,1)
		end
		self.beacon_lockdown = false
	end
end

function PallyPowerWarn:beacon_SetText(beacon_text_time)
	if (self.db.profile.beacon.button) then
		self.beaconInfo:SetText(beacon_text_time)
	else
		self.beaconInfo:SetText(L["Time: "] .. beacon_text_time .. "|n" .. L["Target: "] .. beacon_target)
	end
end

